!  ==============================================================================================================================\n
!  MODULE 	: sechiba
! 
!  CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
!  LICENCE      : IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        Structures the calculation of atmospheric and hydrological 
!! variables by calling diffuco_main, enerbil_main,  hydrol_main,
!! condveg_main and thermosoil_main. Note that sechiba_main
!! calls slowproc_main and thus indirectly calculates the biogeochemical
!! processes as well.
!!
!!\n DESCRIPTION  : :: shumdiag, :: litterhumdiag and :: stempdiag are not 
!! saved in the restart file because at the first time step because they 
!! are recalculated. However, they must be saved as they are in slowproc 
!! which is called before the modules which calculate them.
!! 
!! RECENT CHANGE(S): None 
!! 
!! REFERENCE(S) : None
!!   
!! SVN     :
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
 
MODULE sechiba
 
  USE ioipsl
  USE xios_orchidee
  
  ! modules used :
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE grid
  USE structures
  USE diffuco
  USE condveg
  USE enerbil
  USE hydrol
  USE sechiba_hydrol_arch
  USE thermosoil
  USE sechiba_io_p
  USE slowproc
  USE routing
  USE ioipsl_para
  USE chemistry
  USE stomate_laieff
  USE function_library,  ONLY : cc_to_lai 

  IMPLICIT NONE

  PRIVATE
  PUBLIC sechiba_main, sechiba_initialize, sechiba_clear, &
       sechiba_interface_orchidee_inca

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexveg       !! indexing array for the 3D fields of vegetation
!$OMP THREADPRIVATE(indexveg)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexlai       !! indexing array for the 3D fields of vegetation
!$OMP THREADPRIVATE(indexlai)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexnobio     !! indexing array for the 3D fields of other surfaces (ice,
                                                                     !! lakes, ...)
!$OMP THREADPRIVATE(indexnobio)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexsoil      !! indexing array for the 3D fields of soil types (kjpindex*nstm)
!$OMP THREADPRIVATE(indexsoil)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexgrnd      !! indexing array for the 3D ground heat profiles (kjpindex*ngrnd)
!$OMP THREADPRIVATE(indexgrnd)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexlayer     !! indexing array for the 3D fields of soil layers in CWRR (kjpindex*nslm)
!$OMP THREADPRIVATE(indexlayer)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexnbdl      !! indexing array for the 3D fields of diagnostic soil layers (kjpindex*nbdl)
!$OMP THREADPRIVATE(indexnbdl)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexalb       !! indexing array for the 2 fields of albedo
!$OMP THREADPRIVATE(indexalb)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexsnow      !! indexing array for the 3D fields snow layers
!$OMP THREADPRIVATE(indexsnow)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexcan       !! indexing array for the level fields of the canopy
!$OMP THREADPRIVATE(indexcan)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: jnlevels_loc    !! (JR) number of vegetation levels (integer)
!$OMP THREADPRIVATE(jnlevels_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: veget          !! Fraction of vegetation type (unitless, 0-1)       
!$OMP THREADPRIVATE(veget)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: veget_max      !! Max. fraction of vegetation type (LAI -> infty, unitless)
!$OMP THREADPRIVATE(veget_max)


  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: height         !! Vegetation Height (m)
!$OMP THREADPRIVATE(height)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: totfrac_nobio  !! Total fraction of continental ice+lakes+cities+...
                                                                     !! (unitless, 0-1)
!$OMP THREADPRIVATE(totfrac_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: floodout       !! Flow out of floodplains from hydrol
!$OMP THREADPRIVATE(floodout)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: runoff         !! Surface runoff calculated by hydrol  
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(runoff)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: drainage       !! Deep drainage calculatedd by hydrol  
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(drainage)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: returnflow     !! Water flow from lakes and swamps which returns to 
                                                                     !! the grid box @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(returnflow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: reinfiltration !! Routed water which returns into the soil
!$OMP THREADPRIVATE(reinfiltration)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: irrigation     !! Irrigation flux taken from the routing reservoirs and 
                                                                     !! being put into the upper layers of the soil 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(irrigation)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: emis           !! Surface emissivity (unitless)
!$OMP THREADPRIVATE(emis)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: z0h           !! Surface roughness for heat (m)
!$OMP THREADPRIVATE(z0h)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: z0m           !! Surface roughness for momentum (m)
!$OMP THREADPRIVATE(z0m)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: z0_veg         !! Surface roughness of vegetated part (m) 
!$OMP THREADPRIVATE(z0_veg)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: roughheight    !! Effective height for roughness (m)
!$OMP THREADPRIVATE(roughheight)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: reinf_slope    !! slope coefficient (reinfiltration) 
!$OMP THREADPRIVATE(reinf_slope)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: shumdiag       !! Mean relative soil moisture in the different levels used 
                                                                     !! by thermosoil.f90 (unitless, 0-1)
!$OMP THREADPRIVATE(shumdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: shumdiag_perma !! Saturation degree of the soil 
!$OMP THREADPRIVATE(shumdiag_perma)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: k_litt         !! litter cond.
!$OMP THREADPRIVATE(k_litt)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: litterhumdiag  !! Litter dryness factor (unitless, 0-1)
!$OMP THREADPRIVATE(litterhumdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: stempdiag      !! Temperature which controls canopy evolution (K)
!$OMP THREADPRIVATE(stempdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: qsintveg       !! Water on vegetation due to interception 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(qsintveg)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta2         !! Interception resistance (unitless,0-1)
!$OMP THREADPRIVATE(vbeta2)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta3         !! Vegetation resistance (unitless,0-1)
!$OMP THREADPRIVATE(vbeta3)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta3pot      !! Potential vegetation resistance
!$OMP THREADPRIVATE(vbeta3pot)
!!$ REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbetaco2       !! STOMATE:Vegetation resistance to CO2 (unitless,0-1)
!!$!$OMP THREADPRIVATE(vbetaco2)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: gsmean         !! Mean stomatal conductance for CO2 (mol m-2 s-1)
!$OMP THREADPRIVATE(gsmean)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: cimean         !! STOMATE: mean intercellular CO2 concentration (ppm)
!$OMP THREADPRIVATE(cimean)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vevapwet       !! Interception loss over each PFT 
                                                                     !! @tex $(kg m^{-2} days^{-1})$ @endtex
!$OMP THREADPRIVATE(vevapwet)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir       !! Transpiration @tex $(kg m^{-2} days^{-1})$ @endtex
!$OMP THREADPRIVATE(transpir)




! hydraulic stress variables -------------------------------------

   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir_supply  !! Supply of water for transpiration @tex$$(mm dt^{-1})$ @endtex
!$OMP THREADPRIVATE(transpir_supply)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vir_transpir_supply !! Supply of water for transpiration @tex$$(mm dt^{-1})$ @endtex
!$OMP THREADPRIVATE(vir_transpir_supply)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: stressed      !! Adjusted ecosystem functioning. Takes the unit of the variable
                                                                     !! used as a proxy for waterstress
!$OMP THREADPRIVATE(stressed)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: unstressed    !! Initial ecosystem functioning after the first calculation and 
                                                                     !! before any recalculations. Takes the unit of the variable used
                                                                     !! as a proxy for unstressed.
!$OMP THREADPRIVATE(unstressed)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: e_frac      !! Fraction of water transpired supplied by individual layers (no units)
!$OMP THREADPRIVATE(e_frac)

!$   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vir_transpir_mod !! potential transpiration (transpot) divided by veget_max
!!$!$OMP THREADPRIVATE(vir_transpir_mod)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir_mod      !! transpir divided by veget_max
!$OMP THREADPRIVATE(transpir_mod)

   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_column            !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_column)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_supply_column     !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_supply_column)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_mod_column        !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_mod_column)


   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpot      !! Potential transpiration (needed for irrigation)
!$OMP THREADPRIVATE(transpot)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: qsintmax       !! Maximum amount of water in the canopy interception 
                                                                     !! reservoir @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(qsintmax)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: rveget         !! Surface resistance for the vegetation 
                                                                     !! @tex $(s m^{-1})$ @endtex
!$OMP THREADPRIVATE(rveget)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: rstruct        !! Vegetation structural resistance
!$OMP THREADPRIVATE(rstruct)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: warnings     !! Holds a count of how many warnings we run into
                                                                     !! of different types.  It will get reset at the
                                                                     !! end of each period since we don't wish to restart it.
                                                                     !! Purely a technical diagnostic variable.
!$OMP THREADPRIVATE(warnings)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: snow_nobio     !! Snow mass of non-vegetative surfaces 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(snow_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: snow_nobio_age !! Snow age on non-vegetative surfaces (days)
!$OMP THREADPRIVATE(snow_nobio_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: frac_nobio     !! Fraction of non-vegetative surfaces (continental ice, 
                                                                     !! lakes, ...) (unitless, 0-1)
!$OMP THREADPRIVATE(frac_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: albedo         !! Surface albedo for visible and near-infrared 
                                                                     !! (unitless, 0-1)
!$OMP THREADPRIVATE(albedo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:) :: albedo_pft     !! Albedo (two stream radiation transfer model)
                                                                     !! for visible and near-infrared range 
                                                                     !! for each PFT (unitless)
!$OMP THREADPRIVATE(albedo_pft)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:):: assim_param    !! vcmax, nue, and leaf N for photosynthesis  for photosynthesis
!$OMP THREADPRIVATE(assim_param)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: lai            !! Surface foliaire
!$OMP THREADPRIVATE(lai)
  TYPE(laieff_type), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: laieff_fit   !! The parameters for fitting the effective
                                                                     !! LAI function
!$OMP THREADPRIVATE(laieff_fit)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: gpp            !! STOMATE: GPP. gC/m**2 of total area
!$OMP THREADPRIVATE(gpp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: temp_growth    !! Growth temperature (C) - Is equal to t2m_month 
!$OMP THREADPRIVATE(temp_growth) 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: humrel         !! Relative humidity
!$OMP THREADPRIVATE(humrel)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vegstress      !! Vegetation moisture stress (only for vegetation growth)
!$OMP THREADPRIVATE(vegstress)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:):: frac_age       !! Age efficacity from STOMATE for isoprene 
!$OMP THREADPRIVATE(frac_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: soiltile       !! Fraction of each soil tile (0-1, unitless)
!$OMP THREADPRIVATE(soiltile)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: njsc           !! Index of the dominant soil textural class in the grid cell (1-nscm, unitless)
!$OMP THREADPRIVATE(njsc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta1         !! Snow resistance 
!$OMP THREADPRIVATE(vbeta1)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta4         !! Bare soil resistance
!$OMP THREADPRIVATE(vbeta4)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta5         !! Floodplains resistance
!$OMP THREADPRIVATE(vbeta5)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: soilcap        !!
!$OMP THREADPRIVATE(soilcap)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: soilflx        !!
!$OMP THREADPRIVATE(soilflx)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: temp_sol       !! Soil temperature
!$OMP THREADPRIVATE(temp_sol)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: qsurf          !! near soil air moisture
!$OMP THREADPRIVATE(qsurf)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: flood_res      !! flood reservoir estimate
!$OMP THREADPRIVATE(flood_res)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: flood_frac     !! flooded fraction
!$OMP THREADPRIVATE(flood_frac)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: snow           !! Snow mass [Kg/m^2]
!$OMP THREADPRIVATE(snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: snow_age       !! Snow age
!$OMP THREADPRIVATE(snow_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: drysoil_frac   !! Fraction of visibly (albedo) Dry soil (Between 0 and 1)
!$OMP THREADPRIVATE(drysoil_frac)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evap_bare_lim  !! Bare soil stress
!$OMP THREADPRIVATE(evap_bare_lim)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) ::swc            !! Soil water content (a copy of mc) m3 m-3
!$OMP THREADPRIVATE(swc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) ::ksave          !! Soil conductivity (copy of k mm/d)
!$OMP THREADPRIVATE(ksave)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: co2_flux       !! CO2 flux (gC/m**2 of average ground/s)
!$OMP THREADPRIVATE(co2_flux)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evapot         !! Soil Potential Evaporation
!$OMP THREADPRIVATE(evapot)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evapot_corr    !! Soil Potential Evaporation Correction (Milly 1992)
!$OMP THREADPRIVATE(evapot_corr)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapflo       !! Floodplains evaporation
!$OMP THREADPRIVATE(vevapflo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapsno       !! Snow evaporation
!$OMP THREADPRIVATE(vevapsno)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapnu        !! Bare soil evaporation
!$OMP THREADPRIVATE(vevapnu)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: tot_melt       !! Total melt
!$OMP THREADPRIVATE(tot_melt)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta          !! Resistance coefficient
!$OMP THREADPRIVATE(vbeta)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: rau            !! Density
!$OMP THREADPRIVATE(rau)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: deadleaf_cover !! Fraction of soil covered by dead leaves
!$OMP THREADPRIVATE(deadleaf_cover)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: ptnlev1        !! 1st level Different levels soil temperature
!$OMP THREADPRIVATE(ptnlev1)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: mc_layh        !! Volumetric soil moisture for each layer in hydrol(liquid + ice) (m3/m3)
!$OMP THREADPRIVATE(mc_layh)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: mcl_layh       !! Volumetric soil moisture for each layer in hydrol(liquid) (m3/m3)
!$OMP THREADPRIVATE(mcl_layh)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: tmc_layh       !! Total soil moisture content for each layer in hydrol(liquid + ice) (mm)
!$OMP THREADPRIVATE(tmc_layh)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) ::laieff_isotrop       !! Effective LAI
!$OMP THREADPRIVATE(laieff_isotrop)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) :: Isotrop_Abs_Tot_p   !! Absorbed radiation per level for photosynthesis
!$OMP THREADPRIVATE(Isotrop_Abs_Tot_p)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) :: Isotrop_Tran_Tot_p  !! Transmitted radiation per level for photosynthesis
!$OMP THREADPRIVATE(Isotrop_Tran_Tot_p)

! multi-layer variables --------------------------------
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_rn_grid 
!$OMP THREADPRIVATE(flux_rn_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_h_grid 
!$OMP THREADPRIVATE(flux_h_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_le_grid
!$OMP THREADPRIVATE(flux_le_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: u_speed_grid 
!$OMP THREADPRIVATE(u_speed_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: t_a_next_grid
!$OMP THREADPRIVATE(t_a_next_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: q_a_next_grid 
!$OMP THREADPRIVATE(q_a_next_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: temp_atmos_pres_grid         !! Atmospheric temperature down the column (K) PRESENT STEP
!$OMP THREADPRIVATE(temp_atmos_pres_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: q_atmos_pres_grid            !! Atmospheric specific humididy down the column (kg/kg) PRESENT STEP
!$OMP THREADPRIVATE(q_atmos_pres_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: temp_leaf_pres_grid          !! Leaf temperature down the column (K) PRESENT STEP
!$OMP THREADPRIVATE(temp_leaf_pres_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)      :: u_speed                      !! Canopy wind speed profile
!$OMP THREADPRIVATE(u_speed)
 ! these following two quantities are never calculated explicitly, they always have values passed from
 ! stomate...they need to be saved, though, since stomate is only called once a day
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:,:) :: circ_class_biomass        !! Stem diameter @tex $(m)$ @endtex
!$OMP THREADPRIVATE(circ_class_biomass)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)  :: circ_class_n                 !! Number of trees within each circumference
!$OMP THREADPRIVATE(circ_class_n)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)      :: nlevels_loc                  !! Additional energy to melt snow for snow ablation case (K)


! energy budget/hydraulic stress variables -------------------

  INTEGER, SAVE                                    :: mleb_count = 0               !! count how many times to run mleb_main
!$OMP THREADPRIVATE(mleb_count)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: psold                        !! Old surface dry static energy (J kg^{-1})
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: qsol_sat	                   !! Saturated specific humudity for old temperature 
                                                                                   !! (kg kg^{-1})  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: pdqsold	                   !! Derivative of satured specific humidity at the old 
                                                                                   !! temperature (kg (kg s)^{-1})
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: netrad                       !! Net radiation (W m^{-2})

! ------------------------------------------------------------

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: sum_veget_diff          !! The is the difference between the total
                                                                              !! vegetation fraction in the new and old
                                                                              !! land cover maps.  Useful for land cover
                                                                              !! changes. [-]
!$OMP THREADPRIVATE(sum_veget_diff)

  
                                 !! @tex $(gC m^{-2})$ @endtex
                          
REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:,:)      :: h_array_out       !! heights of tree levels from stomate           
!$OMP THREADPRIVATE(h_array_out)

REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:,:)      :: z_array_out       !! heights of tree levels from stomate           
!$OMP THREADPRIVATE(z_array_out)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)           :: t2m_month_out     !! t2m longterm temperature from stomate           
!$OMP THREADPRIVATE(t2m_month_out)

! new allocatable variables added during merge

 REAL(r_std),ALLOCATABLE, SAVE, DIMENSION (:, :, :)      :: profile_vbeta3
!$OMP THREADPRIVATE(profile_vbeta3)

 REAL(r_std),ALLOCATABLE, SAVE, DIMENSION (:, :, :)      :: profile_rveget
!$OMP THREADPRIVATE(profile_rveget)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)         :: max_height_store     !! Same as z_array, but one less dimension.
                                                                                 !! @tex $(m)$ @endte
!$OMP THREADPRIVATE(max_height_store)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)         :: delta_c13_assim      !! C13 concentration in delta notation
                                                                                 !! @tex $ permille $ @endtex (per thousand)   
!$OMP THREADPRIVATE(delta_c13_assim)
  
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)         :: leaf_ci_out          !! Ci Leaf internal CO2 concentration ()      
!$OMP THREADPRIVATE(leaf_ci_out)
 
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)         :: gpp_day              !! Number of time steps when there is gpp
!$OMP THREADPRIVATE(gpp_day)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)        :: lai_per_level        !! The LAI per vertical level
                                                                                 !! @tex $(m^2 / m^2)$ @endtex
!$OMP THREADPRIVATE(lai_per_level)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)            :: frac_snow_pix        !! The fraction of the whole pixel covered
                                                                                 !! by snow. This is computed from the above
                                                                                 !! two. @tex $-$ @endtex
!$OMP THREADPRIVATE(frac_snow_pix)


  LOGICAL, SAVE                                    :: l_first_sechiba = .TRUE. !! Flag controlling the intialisation (true/false)
!$OMP THREADPRIVATE(l_first_sechiba)

  ! Variables related to snow processes calculations  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: frac_snow_veg   !! Snow cover fraction on vegetation (unitless)  
!$OMP THREADPRIVATE(frac_snow_veg)  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: frac_snow_nobio !! Snow cover fraction on continental ice, lakes, etc (unitless)  
!$OMP THREADPRIVATE(frac_snow_nobio) 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: snowrho      !! snow density for each layer
!$OMP THREADPRIVATE(snowrho)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: snowheat     !! snow heat content for each layer (J/m2)
!$OMP THREADPRIVATE(snowheat)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: snowgrain    !! snow grain size (m)
!$OMP THREADPRIVATE(snowgrain)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: snowtemp     !! snow temperature profile (K)
!$OMP THREADPRIVATE(snowtemp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: snowdz       !! snow layer thickness (m)
!$OMP THREADPRIVATE(snowdz)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: gtemp        !! soil surface temperature
!$OMP THREADPRIVATE(gtemp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)   :: pgflux       !! net energy into snow pack
!$OMP THREADPRIVATE(pgflux)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: cgrnd_snow   !! Integration coefficient for snow numerical scheme
!$OMP THREADPRIVATE(cgrnd_snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: dgrnd_snow   !! Integration coefficient for snow numerical scheme
!$OMP THREADPRIVATE(dgrnd_snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: lambda_snow  !! Coefficient of the linear extrapolation of surface temperature 
                                                                  !! from the first and second snow layers
!$OMP THREADPRIVATE(lambda_snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: temp_sol_add !! Additional energy to melt snow for snow ablation case (K)
!$OMP THREADPRIVATE(temp_sol_add)

  !+++CHECK+++
  ! Variables defined in CN-CAN but no longer present in CN
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)        :: osfcmelt         !! Indicate snow melting in each gridcell
!$OMP THREADPRIVATE(osfcmelt)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: snowflx         !! Snow flux (W m^{-2})
!$OMP THREADPRIVATE(snowflx)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: snowcap         !! Snow calorific capacity (J K^{-1])
!$OMP THREADPRIVATE(snowcap)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: cgrnd_soil      !! matrix coefficient for the computation of soil, from thermosoil
!$OMP THREADPRIVATE(cgrnd_soil)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: dgrnd_soil      !! matrix coefficient for the computation of soil, from thermosoil
!$OMP THREADPRIVATE(dgrnd_soil)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: zdz1_soil       !! numerical constant from thermosoil
!$OMP THREADPRIVATE(zdz1_soil)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: zdz2_soil       !! numerical constant from thermosoil
!$OMP THREADPRIVATE(zdz2_soil)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: albedo_undersnow !! albedo under the snowpack
!$OMP THREADPRIVATE(albedo_undersnow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:) :: pkappa_snow     !! snow thermal conductivity
!$OMP THREADPRIVATE(pkappa_snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: gthick          !! soil surface layer thickness
!$OMP THREADPRIVATE(gthick)

 !+++++++++++

CONTAINS

!!  =============================================================================================================================
!! SUBROUTINE:    sechiba_initialize
!!
!>\BRIEF	  Initialize all prinicipal modules by calling their "_initialize" subroutines
!!
!! DESCRIPTION:	  Initialize all prinicipal modules by calling their "_initialize" subroutines
!!
!! \n
!_ ==============================================================================================================================

  SUBROUTINE sechiba_initialize( &
       kjit,         kjpij,        kjpindex,     index,                   &
       lalo,         contfrac,     neighbours,   resolution, zlev,        &
       u,            v,            qair,         t2m,        temp_air,    &
       petAcoef,     peqAcoef,     petBcoef,     peqBcoef,                &
       precip_rain,  precip_snow,  lwdown,       swnet,      swdown,      &
       pb,           rest_id,      hist_id,      hist2_id,                &
       rest_id_stom, hist_id_stom, hist_id_stom_IPCC,                     &
       coastalflow,  riverflow,    tsol_rad,     vevapp,       qsurf_out, &
       z0m_out,      z0h_out,      albedo,       fluxsens,     fluxlat,      emis_out,  &
       netco2flux,   fco2_lu,      temp_sol_new, tq_cdrag, coszang)

!! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                               :: kjit              !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpij             !! Total size of the un-compressed grid 
                                                                                  !! (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpindex          !! Domain size - terrestrial pixels only 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id           !! _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id           !! _History_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist2_id          !! _History_ file 2 identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id_stom      !! STOMATE's _Restart_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id_stom      !! STOMATE's _History_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT(in)                                :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file file 
                                                                                  !! identifier (unitless)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)          :: lalo              !! Geographic coordinates (latitude,longitude)
                                                                                  !! for grid cells (degrees)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: contfrac          !! Fraction of continent in the grid 
                                                                                  !! (unitless, 0-1)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index             !! Indices of the pixels on the map. 
                                                                                  !! Sechiba uses a reduced grid excluding oceans
                                                                                  !! ::index contains the indices of the 
                                                                                  !! terrestrial pixels only! (unitless)
    INTEGER(i_std), DIMENSION (kjpindex,NbNeighb), INTENT(in):: neighbours        !! Neighboring grid points if land!(unitless)
    REAL(r_std), DIMENSION (kjpindex,2), INTENT(in)          :: resolution        !! Size in x and y of the grid (m)
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u                 !! Lowest level wind speed in direction u 
                                                                                  !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: v                 !! Lowest level wind speed in direction v 
                                                                                  !! @tex $(m.s^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev              !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair              !! Lowest level specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: t2m               !! 2m air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_rain       !! Rain precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_snow       !! Snow precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown            !! Down-welling long-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet             !! Net surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swdown            !! Down-welling surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_air          !! Air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef          !! Coefficients A for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef          !! Coefficients A for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef          !! Coefficients B for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef          !! Coefficients B for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb                !! Surface pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: coszang           !! cosine of solar zenith angle


!! 0.2 Output variables
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: coastalflow       !! Outflow on coastal points by small basins.
                                                                                  !! This is the water which flows in a disperse 
                                                                                  !! way into the ocean
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: riverflow         !! Outflow of the major rivers.
                                                                                  !! The flux will be located on the continental 
                                                                                  !! grid but this should be a coastal point  
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad          !! Radiative surface temperature 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp            !! Total of evaporation 
                                                                                  !! @tex $(kg m^{-2} days^{-1})$ @endtex
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf_out         !! Surface specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: z0m_out           !! Surface roughness momentum (output diagnostic, m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: z0h_out           !! Surface roughness heat (output diagnostic, m)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (out)         :: albedo            !! Surface albedo for visible and near-infrared
                                                                                  !! (unitless, 0-1)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsens          !! Sensible heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxlat           !! Latent heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: emis_out          !! Emissivity (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: netco2flux        !! Sum CO2 flux over PFTs 
                                                                                  !! ??(gC m^{-2} s^{-1})??
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fco2_lu           !! Land Cover Change CO2 flux 
                                                                                  !! ??(gC m^{-2} s^{-1})??
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: temp_sol_new      !! New ground temperature (K)

!! 0.3 Modified
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: tq_cdrag          !! Surface drag coefficient (-) 

!! 0.4 Local variables
    INTEGER(i_std)                                           :: ji, jv, ilev      !! Index (unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: histvar           !! Computations for history files (unitless)
    CHARACTER(LEN=80)                                        :: var_name          !! To store variables names for I/O (unitless)
    REAL(r_std),DIMENSION (kjpindex)                         :: epot_air          !! Air potential energy (??J)
    REAL(r_std),DIMENSION(nlevels_tot)                       :: Collim_Abs_Tot    !! collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                       :: Collim_Alb_Tot    !! Collimated (direct) total albedo for a given level
    REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm)         :: laieff_collim     !! Leaf Area Index Effective for direct light
    INTEGER(i_std)                                           :: init_config       !! Identifer of the configuration used to 
                                                                                  !! initialize stomate/or sechiba

!_ ================================================================================================================================

    IF (printlev>=3) WRITE(numout,*) ' sechiba kjpindex =',kjpindex

    !! 1. Initialize variables on first call
    
    !! 1.2 Initialize most of sechiba's variables
    CALL sechiba_init (kjit, kjpij, kjpindex, index, rest_id, lalo)
    
    !! 1.3 Initialize stomate's variables
    CALL slowproc_initialize (kjit,          kjpij,        kjpindex,                          &
                              rest_id,       rest_id_stom, hist_id_stom,   hist_id_stom_IPCC, &
                              index,         indexveg,     lalo,           neighbours,        &
                              resolution,    contfrac,     t2m,                               &
                              soiltile,      reinf_slope,  deadleaf_cover, assim_param,       &
                              frac_age,      height,       veget,             &
                              frac_nobio,    njsc,         veget_max,      tot_bare_soil,     &       
                              totfrac_nobio, qsintmax,     co2_flux,       fco2_lu,           &
                              temp_growth,   circ_class_biomass,   &
                              circ_class_n,  lai_per_level,laieff_fit,     h_array_out,       &
                              z_array_out,   max_height_store)


    netco2flux(:) = zero
    DO jv = 2,nvm
       netco2flux(:) = netco2flux(:) + co2_flux(:,jv)*veget_max(:,jv)
    ENDDO

    !! 1.4 Initialize diffusion coefficients
    CALL diffuco_initialize (kjit,    kjpindex, index,                  &
                             rest_id, lalo,     neighbours, resolution, &
                             rstruct, tq_cdrag)
    
    !! 1.5 Initialize remaining variables of energy budget
    CALL enerbil_initialize (kjit,     kjpindex,     index,    rest_id,  &
                             qair,                                       &
                             temp_sol, temp_sol_new, tsol_rad,           &
                             evapot,   evapot_corr,  qsurf,    fluxsens, &
                             fluxlat,  vevapp )

    !! 1.5 Initialize remaining variables of the multi-layer energy budget
    ! CALL mleb_main

    IF (ok_mleb) THEN
       CALL mleb_initialize

    END IF ! (ok_mleb)

    !! do we need to call enerbill here... it is called in enerbil_main, but we might needs these variables
    !! for the multi-layer energy budget. Maybe we should put all this in the ok_mleb if statement? asla, MERGE
    !! it crashes... investigate further
    !!CALL enerbil_begin (kjpindex, temp_sol, lwdown, swnet, pb, psold, qsol_sat, pdqsold, netrad, emis)

       ! ---------------------------------------------------------------------------------

       ! Initialise some variables for the multilayer energy budget
       ! The temperature and specific humidity column for the first time step must also
       ! have an initial value
       ! The leaf temperature profile for the first time step (for the multi-level energy budget) 
       ! must have an initial value, which we also set to the temperature of air (just for this
       ! initial step)
       DO ji = 1, kjpindex
          temp_atmos_pres_grid(ji,:) = temp_air(ji)
          q_atmos_pres_grid(ji,:) = qair(ji)
          temp_leaf_pres_grid(ji,:) = temp_air(ji)     
       END DO ! i = 1, kjpindex

       ! The sechiba_hydrol_arch requires initialisation for swc, the soil water content.
       swc(:,:,:) = 0.0d0
       ksave(:,:,:)=min_sechiba

       ! The 'ok_impose_can_structure' flag activates the sections of code which directly
       ! link the energy budget scheme to the the size and LAI profile of the canopy for
       ! the respective PFT and age class that is calculated in stomate, for the albedo.
       ! If 'ok_impose_can_structure' is TRUE, and jnlvls > 1, then the model takes LAI
       ! profile information and canopy level heights from the run.def.
       ! If 'ok_impose_can_structure' is FALSE, and jnlvls > 1, then the profile infomation
       ! and canopy levels heights comes from the PGap-based processes for calculation of
       ! stand profile information in stomate.
       ! If jnlvls=1, then the code behaves as for the single layer model.
       ! Should we put this into the mleb if statement as well?? asla, MERGE

       IF (.NOT. ok_impose_can_structure) THEN   
     
		   ! for enerbil nextstep initialisation
	   
		   ! we have a problem in that z_array_out has a value of zero on the first call 
		   !   of slowproc_main (above), so we define z_array_out with temporary values.

		   u_speed(:) = 0.0d0		   
		   z_array_out(:,:,:,:) = 0.0d0		   
		   h_array_out = 0.0d0                   		   
		   collim_abs_tot(:) = 0.0d0
		   Collim_Alb_Tot(:) = 0.0d0

           ! Read in the initial maximum canopy height
           !Config Key   = MAX_HEIGHT_STORE
           !Config Desc  = 
           !Config If    = OK_NEW_ENERBIL_NEXTSTEP
           !Config Def   = 0.0
           !Config Help  = 
           !Config Units = [-]
		   max_height_store(:,:) = 20.0d0
           CALL getin_p('MAX_HEIGHT_STORE', max_height_store(:,:))

       
           DO ilev = 1, nlevels_tot
         
               z_array_out(:,:,:,ilev) = (REAL(ilev)-1.0d0) * (max_height_store(1,1)/REAL(nlevels_tot))

           END DO ! i = 1, nlevels_tot
		           
      END IF ! IF (ok_impose_can_structure) 


    ! ---------------------------------------------------------------------------------


    !! 1.7 Initialize remaining hydrological variables
    CALL hydrol_initialize ( kjit,           kjpindex,  index,         rest_id,          &
         njsc,           soiltile,  veget,         veget_max,        &
         humrel,         vegstress, drysoil_frac,                    &
         shumdiag_perma, qsintveg,                                   &
         evap_bare_lim,  snow,      snow_age,      snow_nobio,       &
         snow_nobio_age, snowrho,   snowtemp,      snowgrain,        &
         snowdz,         snowheat,  &
         mc_layh,    mcl_layh,  tmc_layh)
    

    !! 1.9 Initialize surface parameters (emissivity, albedo and roughness)
    CALL condveg_initialize (kjit, kjpindex, index, rest_id, &
         lalo, neighbours, resolution, contfrac, &
         veget, veget_max, frac_nobio, totfrac_nobio, &
         zlev, snow, snow_age, snow_nobio, snow_nobio_age, &
         drysoil_frac, height, snowdz,snowrho, tot_bare_soil, &
         temp_air, pb, u, v, &
         circ_class_biomass, circ_class_n, &
         emis, albedo, z0m, z0h, roughheight, &
         frac_snow_veg,frac_snow_nobio, &
         coszang, &!z0_veg, &
         Isotrop_Abs_Tot_p, Isotrop_Tran_Tot_p, laieff_fit, albedo_pft, &
         Collim_Abs_Tot, Collim_Alb_Tot, laieff_collim, laieff_isotrop)
    
    !! 1.10 Initialization of soil thermodynamics
    
    CALL thermosoil_initialize (kjit, kjpindex, rest_id,  &
         temp_sol_new, snow,       shumdiag_perma,        &
         soilcap,      soilflx,    stempdiag,             &
         gtemp,               &
         mc_layh,  mcl_layh,   tmc_layh,       njsc ,     &
         frac_snow_veg,frac_snow_nobio,totfrac_nobio,     &
         snowdz, snowrho, snowtemp, lambda_snow, cgrnd_snow, dgrnd_snow, pb)


    !! 1.12 Initialize river routing
    IF ( river_routing .AND. nbp_glo .GT. 1) THEN
       !! 1.12.1 Initialize river routing
       CALL routing_initialize( kjit,        kjpindex,       index,                 &
            rest_id,     hist_id,        hist2_id,   lalo,      &
            neighbours,  resolution,     contfrac,   stempdiag, &
            returnflow,  reinfiltration, irrigation, riverflow, &
            coastalflow, flood_frac,     flood_res )
    ELSE
       !! 1.12.2 No routing, set variables to zero
       riverflow(:) = zero
       coastalflow(:) = zero
       returnflow(:) = zero
       reinfiltration(:) = zero
       irrigation(:) = zero
       flood_frac(:) = zero
       flood_res(:) = zero

       CALL xios_orchidee_send_field("coastalflow",coastalflow/dt_sechiba)  
       CALL xios_orchidee_send_field("riverflow",riverflow/dt_sechiba)  

    ENDIF
    
    !! 1.13 Write internal variables to output fields
    z0m_out(:) = z0m(:)
    z0h_out(:) = z0h(:)
    emis_out(:) = emis(:) 
    qsurf_out(:) = qsurf(:)
    
    !+++CHECK+++
    !! 1.14 Set lai
    !! I am not sure we need this at all?? asla, MERGE
    lai(:,ibare_sechiba) = zero
    DO jv = 2, nvm
       DO ji = 1, kjpindex
          lai(ji,jv) = cc_to_lai(circ_class_biomass(ji,jv,:,ileaf,icarbon),circ_class_n(ji,jv,:),jv)
       ENDDO
    ENDDO
    !+++++++++++
    
  END SUBROUTINE sechiba_initialize

!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_main
!!
!>\BRIEF        Main routine for the sechiba module performing three functions:
!! calculating temporal evolution of all variables and preparation of output and 
!! restart files (during the last call only)
!!
!!\n DESCRIPTION : Main routine for the sechiba module. 
!! One time step evolution consists of:
!! - call sechiba_var_init to do some initialization,
!! - call slowproc_main to do some daily calculations
!! - call diffuco_main for diffusion coefficient calculation,
!! - call enerbil_main for energy budget calculation,
!! - call condveg_main for surface conditions such as roughness, albedo, and emmisivity,
!! - call thermosoil_main for soil thermodynamic calculation,
!! - call sechiba_end to swap previous to new fields.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): Hydrological variables (:: coastalflow and :: riverflow),
!! components of the energy budget (:: tsol_rad, :: vevapp, :: fluxsens, 
!! :: temp_sol_new and :: fluxlat), surface characteristics (:: z0_out, :: emis_out, 
!! :: tq_cdrag and :: albedo) and land use related CO2 fluxes (:: netco2flux and 
!! :: fco2_lu)            
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale = 0.5]{sechibamainflow.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE sechiba_main (kjit, kjpij, kjpindex, index, &
       & ldrestart_read, ldrestart_write, &
       & lalo, contfrac, neighbours, resolution,&
       & zlev, u, v, qair, q2m, t2m, temp_air, epot_air, ccanopy, &
       & tq_cdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
       & precip_rain, precip_snow, lwdown, swnet, swdown, coszang, pb, &
       & vevapp, fluxsens, fluxlat, coastalflow, riverflow, netco2flux, fco2_lu, &
       & tsol_rad, temp_sol_new, qsurf_out, albedo, emis_out, z0m_out, z0h_out,&
       & rest_id, hist_id, hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC)

!! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                               :: kjit              !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpij             !! Total size of the un-compressed grid 
                                                                                  !! (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpindex          !! Domain size - terrestrial pixels only 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id           !! _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id           !! _History_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist2_id          !! _History_ file 2 identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id_stom      !! STOMATE's _Restart_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id_stom      !! STOMATE's _History_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT(in)                                :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file file 
                                                                                  !! identifier (unitless)
    LOGICAL, INTENT(in)                                      :: ldrestart_read    !! Logical for _restart_ file to read 
                                                                                  !! (true/false)
    LOGICAL, INTENT(in)                                      :: ldrestart_write   !! Logical for _restart_ file to write 
                                                                                  !! (true/false)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)          :: lalo              !! Geographic coordinates (latitude,longitude)
                                                                                  !! for grid cells (degrees)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: contfrac          !! Fraction of continent in the grid 
                                                                                  !! (unitless, 0-1)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index             !! Indices of the pixels on the map. 
                                                                                  !! Sechiba uses a reduced grid excluding oceans
                                                                                  !! ::index contains the indices of the 
                                                                                  !! terrestrial pixels only! (unitless)
    INTEGER(i_std), DIMENSION(kjpindex,NbNeighb), INTENT(in) :: neighbours        !! Neighboring grid points if land!(unitless)
    REAL(r_std), DIMENSION (kjpindex,2), INTENT(in)          :: resolution        !! Size in x and y of the grid (m)
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u                 !! Lowest level wind speed in direction u 
                                                                                  !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: v                 !! Lowest level wind speed in direction v 
                                                                                  !! @tex $(m.s^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev              !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair              !! Lowest level specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q2m               !! 2m specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: t2m               !! 2m air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_rain       !! Rain precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_snow       !! Snow precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown            !! Down-welling long-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: coszang           !! Cosine of the solar zenith angle (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet             !! Net surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swdown            !! Down-welling surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_air          !! Air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air          !! Air potential energy (??J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: ccanopy           !! CO2 concentration in the canopy (ppm)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef          !! Coefficients A for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef          !! Coefficients A for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef          !! Coefficients B for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef          !! Coefficients B for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb                !! Surface pressure (hPa)


!! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: coastalflow       !! Outflow on coastal points by small basins.
                                                                                  !! This is the water which flows in a disperse 
                                                                                  !! way into the ocean
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: riverflow         !! Outflow of the major rivers.
                                                                                  !! The flux will be located on the continental 
                                                                                  !! grid but this should be a coastal point  
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad          !! Radiative surface temperature 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp            !! Total of evaporation 
                                                                                  !! @tex $(kg m^{-2} days^{-1})$ @endtex
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf_out         !! Surface specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: z0m_out           !! Surface roughness momentum (output diagnostic, m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: z0h_out           !! Surface roughness heat (output diagnostic, m)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (out)         :: albedo            !! Surface albedo for visible and near-infrared
                                                                                  !! (unitless, 0-1)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsens          !! Sensible heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxlat           !! Latent heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: emis_out          !! Emissivity (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: netco2flux        !! Sum CO2 flux over PFTs 
                                                                                  !! ??(gC m^{-2} s^{-1})??
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fco2_lu           !! Land Cover Change CO2 flux 
                                                                                  !! ??(gC m^{-2} s^{-1})??

!! 0.3 Modified

    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: tq_cdrag          !! Surface drag coefficient (-) 
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: temp_sol_new      !! New ground temperature (K)

!! 0.4 local variables

    INTEGER(i_std)                                           :: ji, jv, ilevel    !! Index (unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: histvar           !! Computations for history files (unitless)
    CHARACTER(LEN=80)                                        :: var_name          !! To store variables names for I/O (unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_treefrac      !! Total fraction occupied by trees (0-1, uniless) 
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_grassfrac     !! Total fraction occupied by grasses (0-1, unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_cropfrac      !! Total fraction occcupied by crops (0-1, unitess)
    REAL(r_std),DIMENSION (kjpindex,nvm)                     :: tmc_pft           !! Total soil water per PFT (mm/m2) 
    REAL(r_std),DIMENSION (kjpindex,nvm)                     :: drainage_pft     !! Drainage per PFT (mm/m2)   
    REAL(r_std),DIMENSION (kjpindex,nvm)                     :: swc_pft          !! Relative Soil water content [tmcr:tmcs] per pft (-)    
    REAL(r_std), DIMENSION(kjpindex)                         :: grndflux          !! Net energy into soil (W/m2)
    REAL(r_std), DIMENSION(kjpindex,nsnow)                   :: snowliq           !! Liquid water content (m)
    REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm)        :: laieff_collim      !! Leaf Area Index Effective for direct light
    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Abs_Tot     !! Collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Alb_Tot     !! Collimated (direct) total albedo for a given level

REAL(r_std), SAVE                                       :: temp_surf_pres        !! temperature at surface for the present timestep (K)
!$OMP THREADPRIVATE(temp_surf_pres)
    REAL(r_std), SAVE                                       :: temp_surf_next        !! temperature at surface for the next timestep (K)
!$OMP THREADPRIVATE(temp_surf_next)
    INTEGER, SAVE                                           :: james_step_count      !! $OMP THREADPRIVATE(james_step_count)
!$OMP THREADPRIVATE(james_step_count)
    REAL(r_std),DIMENSION (kjpindex)                        :: netrad_out            !! Net radiation flux add by YC 20140313

! (for the merge)

   REAL(r_std), DIMENSION(kjpindex)                        :: vbeta2sum             ! sum of vbeta2 coefficients across all PFTs (-)
   REAL(r_std), DIMENSION(kjpindex)                        :: vbeta3sum             ! sum of vbeta3 coefficients across all PFTs (-)
   REAL(r_std),DIMENSION (kjpindex,nvm)                    :: vbeta23               !! Beta for fraction of wetted foliage that will
                                                                                    !! transpire once intercepted water has evaporated (-)
   REAL(r_std), DIMENSION (kjpindex,nvm,nlai)              :: leaf_ci               !! intercellular CO2 concentration (ppm)
   REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gs_distribution
   REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gs_diffuco_output
   REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gstot_component
   REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gstot_frac
   REAL(r_std), DIMENSION (nlevels_tot)                    :: u_speed               !! canopy wind speed profile


   REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_vbeta3
   REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_rveget

   REAL(r_std)                                             :: flux_ground_h            ! sensible heat flux at surface (W^{m^2})
   REAL(r_std)                                             :: flux_ground_le           ! latent heat flux at surface (W^{m^2})

   INTEGER                                                 :: ilev,ivm,ipts  

!_ ================================================================================================================================

    IF (printlev>=3) WRITE(numout,*) ' sechiba kjpindex =',kjpindex
    !! Initialize local printlev
    !printlev_loc=get_printlev('sechiba')

    !! 1. Initialize variables at each time step
    CALL sechiba_var_init (kjpindex, rau, pb, temp_air) 

    !! 2. Compute diffusion coefficients
    CALL diffuco_main (kjit, kjpindex, &
         & index, indexveg, indexlai, u, v, &
         & zlev, z0m, z0h, roughheight, temp_sol, temp_air, &
         & temp_growth, rau, tq_cdrag, qsurf, qair, &
         & q2m, t2m, pb, evap_bare_lim, &
         & evapot, evapot_corr, snow, flood_frac, flood_res, &
         & frac_nobio, snow_nobio, totfrac_nobio, swnet, swdown, &
         & coszang, ccanopy, humrel, vegstress, veget, &
         & veget_max, circ_class_biomass, circ_class_n, qsintveg, qsintmax, assim_param, &
         & vbeta, vbeta1, vbeta2, vbeta3, &
         & vbeta3pot, vbeta4, vbeta5, gsmean, rveget, &
         & rstruct, cimean, gpp, lalo, neighbours, &
         & resolution, ptnlev1, precip_rain, frac_age, tot_bare_soil, &
         & frac_snow_veg, frac_snow_nobio, &
         & hist_id, hist2_id, vbeta2sum, vbeta3sum, Isotrop_Abs_Tot_p, &
         & Isotrop_Tran_Tot_p, lai_per_level, vbeta23, leaf_ci, &
         & gs_distribution, gs_diffuco_output, gstot_component, gstot_frac,&
         & warnings, u_speed, profile_vbeta3, profile_rveget, &
         & delta_c13_assim, leaf_ci_out)

    IF ( ok_c13 ) THEN 
                 
      ! When there is no photosynthesis, carbon isotopic values is not calulated.  
      ! Because daily mean of carbon isotopic value in ORCHIDEE contained night values,  
      ! to avoide biased daily mean value by zeros, daily mean of carbon isotopic  
      ! discriminations should be calculated only when there is photosynthesis 
      
      gpp_day(:,:) = zero 
      
      DO ipts = 1,kjpindex 
        DO jv = 1,nvm 
          IF (gpp(ipts,jv) .LT. min_sechiba) THEN 
            gpp_day(ipts,jv) = zero 
      
          ELSE 
            gpp_day(ipts,jv) = gpp_day(ipts,jv) + 1 
           
          END IF 
        END DO 
      END DO 
    END IF 

  !  !! 3. Compute energy balance
  IF (ok_mleb) THEN

        IF (printlev_loc>=4)THEN
           DO ivm = 1,nvm
              WRITE(numout,*) 'laieff_isotrop sechiba_main  for ivm', &
              ivm,'is', laieff_isotrop(:,:,ivm)
           ENDDO
        ENDIF


        CALL sechiba_mleb_hs( &
            kjit,         kjpij,        kjpindex,     index,                   &
            lalo,         contfrac,     neighbours,   resolution, zlev,        &
            u,            v,            qair,         t2m,        temp_air,    &
            petAcoef,     peqAcoef,     petBcoef,     peqBcoef,                &
            precip_rain,  precip_snow,  lwdown,       swnet,      swdown,      &
            pb,           rest_id,      hist_id,      hist2_id,                &
            rest_id_stom, hist_id_stom, hist_id_stom_IPCC,                     &
            fluxsens,     fluxlat,      temp_sol_new, tsol_rad,   vevapp,      &
            vbeta2sum,    vbeta3sum,    tq_cdrag,   ldrestart_read,            &
            ldrestart_write, epot_air,  flux_ground_h, flux_ground_le,         & 
            temp_surf_pres, temp_surf_next, coszang, q2m, ccanopy, vbeta23,    &
            leaf_ci, gs_distribution,   gs_diffuco_output, gstot_component,    &
            gstot_frac)
   ELSE ! (ok_mleb)
        CALL enerbil_main (kjit, kjpindex, &
            & index, indexveg, zlev, lwdown, swnet, epot_air, temp_air, u, v, &
            & petAcoef, petBcoef, &
            & qair, peqAcoef, peqBcoef, pb, rau, vbeta, vbeta1, &
            & vbeta2, vbeta3, vbeta3pot, vbeta4, vbeta5, &
            & emis, soilflx, soilcap, tq_cdrag, humrel, fluxsens, fluxlat, &
            & vevapp, transpir, transpot, vevapnu, vevapwet, vevapsno, vevapflo, &
            & temp_sol, tsol_rad, &
            & temp_sol_new, qsurf, evapot, evapot_corr, rest_id, hist_id, hist2_id, &
            & precip_rain,  pgflux, snowdz, temp_sol_add)

  END IF ! (ok_mleb)

  !! 4. Compute hydrology
  !! 4.1 Water balance from CWRR module (11 soil layers)
  CALL hydrol_main (kjit, kjpindex, &
       & index, indexveg, indexsoil, indexlayer, indexnbdl, &
       & temp_sol_new, floodout, runoff, drainage, frac_nobio, totfrac_nobio, &
       & vevapwet, veget, veget_max, njsc, &
       & qsintmax, qsintveg, vevapnu, vevapsno, vevapflo, snow, snow_age, &
       & snow_nobio, snow_nobio_age,  &
       & tot_melt, transpir, precip_rain, precip_snow, returnflow, &
       & reinfiltration, irrigation, &
       & humrel, vegstress, drysoil_frac, evapot, evapot_corr, evap_bare_lim, &
       & flood_frac, flood_res, &
       & shumdiag,shumdiag_perma, k_litt, litterhumdiag, soilcap, soiltile, reinf_slope,&
       & rest_id, hist_id, hist2_id,&
       & stempdiag, &
       & temp_air, pb, u, v, tq_cdrag, swnet, pgflux, &
       & snowrho, snowtemp, snowgrain, snowdz, snowheat, snowliq, &
       & grndflux, gtemp, tot_bare_soil, &
       & lambda_snow, cgrnd_snow, dgrnd_snow, temp_sol_add, &
       & mc_layh, mcl_layh, tmc_layh, tmc_pft, drainage_pft, swc_pft, swc, ksave, &
       & e_frac)

                
    !! 6. Compute surface variables (emissivity, albedo and roughness)
    CALL condveg_main (kjit, kjpindex, index, rest_id, hist_id, hist2_id, &
         lalo, neighbours, resolution, contfrac, &
         veget, veget_max, frac_nobio, totfrac_nobio, &
         zlev, snow, snow_age, snow_nobio, snow_nobio_age, &
         drysoil_frac, height, snowdz, snowrho, tot_bare_soil, &
         temp_air, pb, u, v, &
         circ_class_biomass, circ_class_n,&
         emis, albedo, z0m, z0h, roughheight, &
         frac_snow_veg, frac_snow_nobio, coszang, &! z0_veg, & ! fromthe MERGE
         Isotrop_Abs_Tot_p, Isotrop_Tran_Tot_p, laieff_fit, albedo_pft, &
         Collim_Abs_Tot, Collim_Alb_Tot, laieff_collim, laieff_isotrop)

        IF (printlev_loc>=4)THEN
           DO ivm = 1,nvm
              WRITE(numout,*) 'laieff_isotrop after condveg_main in sechiba_main  for ivm', &
              ivm,'is', laieff_isotrop(:,:,ivm)
           ENDDO
        ENDIF

    !! 7. Compute soil thermodynamics
    CALL thermosoil_main (kjit, kjpindex, &
         index, indexgrnd, &
         temp_sol_new, snow, soilcap, soilflx, &
         shumdiag_perma, stempdiag, ptnlev1, rest_id, hist_id, hist2_id, &
         snowdz,snowrho,snowtemp,gtemp,pb,&
         mc_layh, mcl_layh, tmc_layh, njsc, frac_snow_veg, &
         frac_snow_nobio, totfrac_nobio, temp_sol_add, &
         lambda_snow, cgrnd_snow, dgrnd_snow)


    !! 8. Compute river routing 
    IF ( river_routing .AND. nbp_glo .GT. 1) THEN
       !! 8.1 River routing
       CALL routing_main (kjit, kjpindex, index, &
            & lalo, neighbours, resolution, contfrac, totfrac_nobio, veget_max, floodout, runoff, &
            & drainage, transpot, precip_rain, humrel, k_litt, flood_frac, flood_res, &
            & stempdiag, reinf_slope, returnflow, reinfiltration, irrigation, riverflow, coastalflow, rest_id, hist_id, hist2_id)
    ELSE
       !! 8.2 No routing, set variables to zero
       riverflow(:) = zero
       coastalflow(:) = zero
       returnflow(:) = zero
       reinfiltration(:) = zero
       irrigation(:) = zero
       flood_frac(:) = zero
       flood_res(:) = zero
    ENDIF

    !! 9. Compute slow processes (i.e. 'daily' and annual time step)
    CALL slowproc_main (kjit, kjpij, kjpindex, &
         index, indexveg, lalo, neighbours, &
         resolution, contfrac, soiltile,  &
         temp_air, temp_sol, stempdiag, vegstress, &
         shumdiag, litterhumdiag, precip_rain, precip_snow, &
         pb, gpp, tmc_pft, drainage_pft, swc_pft, &
         deadleaf_cover, assim_param, frac_age, &
         height, veget, frac_nobio, &
         veget_max, totfrac_nobio, qsintmax, rest_id, &
         hist_id, hist2_id, rest_id_stom, hist_id_stom, &
         hist_id_stom_IPCC, co2_flux, fco2_lu, temp_growth, & 
         tot_bare_soil, circ_class_biomass, circ_class_n, &
         lai_per_level, max_height_store, laieff_fit, &
         h_array_out, z_array_out, transpir, transpir_mod, &
         transpir_supply, vir_transpir_supply, coszang, &
         stressed, unstressed, isotrop_tran_tot_p, &
         u, v)

    !#481
    !WRITE(numout,*) 'AHAAA: after slowproc_main veget_max', veget_max(:,1)

    !! 9.2 Compute global CO2 flux
    netco2flux(:) = zero
    DO jv = 2,nvm
       netco2flux(:) = netco2flux(:) + co2_flux(:,jv)*veget_max(:,jv)
    ENDDO
 
    !! 10. Update the temperature (temp_sol) with newly computed values
    CALL sechiba_end (kjpindex, temp_sol_new, temp_sol)

    !! 11. Write internal variables to output fields
    z0m_out(:) = z0m(:)
    z0h_out(:) = z0h(:)
    emis_out(:) = emis(:)
    qsurf_out(:) = qsurf(:)
 
    !! 12. Write global variables to history files
    sum_treefrac(:) = zero
    sum_grassfrac(:) = zero
    sum_cropfrac(:) = zero
    DO jv = 2, nvm 
       IF (is_tree(jv) .AND. natural(jv)) THEN
          sum_treefrac(:) = sum_treefrac(:) + veget_max(:,jv)
       ELSE IF ((.NOT. is_tree(jv))  .AND. natural(jv)) THEN
          sum_grassfrac(:) = sum_grassfrac(:) + veget_max(:,jv)
       ELSE 
          sum_cropfrac = sum_cropfrac(:) + veget_max(:,jv)
       ENDIF
    ENDDO          

    CALL xios_orchidee_send_field("evapnu",vevapnu*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("snow",snow)
    CALL xios_orchidee_send_field("snowage",snow_age)
    CALL xios_orchidee_send_field("snownobio",snow_nobio)
    CALL xios_orchidee_send_field("snownobioage",snow_nobio_age)
    CALL xios_orchidee_send_field("frac_snow", SUM(frac_snow_nobio,2)*totfrac_nobio+frac_snow_veg*(1-totfrac_nobio))
    CALL xios_orchidee_send_field("frac_snow_veg", frac_snow_veg)
    CALL xios_orchidee_send_field("frac_snow_nobio", frac_snow_nobio)
    CALL xios_orchidee_send_field("pgflux",pgflux)
    CALL xios_orchidee_send_field("reinf_slope",reinf_slope)
    CALL xios_orchidee_send_field("njsc",REAL(njsc, r_std))
    CALL xios_orchidee_send_field("vegetfrac",veget)
    CALL xios_orchidee_send_field("maxvegetfrac",veget_max)
    CALL xios_orchidee_send_field("nobiofrac",frac_nobio)
    CALL xios_orchidee_send_field("soiltile",soiltile)
    CALL xios_orchidee_send_field("rstruct",rstruct)
    CALL xios_orchidee_send_field("gpp",gpp/dt_sechiba)
    CALL xios_orchidee_send_field("nee",co2_flux/dt_sechiba)
    CALL xios_orchidee_send_field("drysoil_frac",drysoil_frac)
    CALL xios_orchidee_send_field("evapflo",vevapflo*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("evapflo_alma",vevapflo/dt_sechiba)
    CALL xios_orchidee_send_field("k_litt",k_litt)
    CALL xios_orchidee_send_field("beta",vbeta)
    CALL xios_orchidee_send_field("vbeta1",vbeta1)
    CALL xios_orchidee_send_field("vbeta2",vbeta2)
    CALL xios_orchidee_send_field("vbeta3",vbeta3)
    CALL xios_orchidee_send_field("vbeta4",vbeta4)
    CALL xios_orchidee_send_field("vbeta5",vbeta5)
    CALL xios_orchidee_send_field("gsmean",gsmean)
    CALL xios_orchidee_send_field("cimean",cimean)
    CALL xios_orchidee_send_field("rveget",rveget)
 
    histvar(:)=SUM(vevapwet(:,:),dim=2)
    CALL xios_orchidee_send_field("evspsblveg",histvar/dt_sechiba)
    histvar(:)= vevapnu(:)+vevapsno(:)
    CALL xios_orchidee_send_field("evspsblsoi",histvar/dt_sechiba)
    histvar(:)=SUM(transpir(:,:),dim=2)
    CALL xios_orchidee_send_field("tran",histvar/dt_sechiba)
    histvar(:)= sum_treefrac(:)*100*contfrac(:)
    CALL xios_orchidee_send_field("treeFrac",histvar)
    histvar(:)= sum_grassfrac(:)*100*contfrac(:)
    CALL xios_orchidee_send_field("grassFrac",histvar)
    histvar(:)= sum_cropfrac(:)*100*contfrac(:)
    CALL xios_orchidee_send_field("cropFrac",histvar)
    histvar(:)=veget_max(:,1)*100*contfrac(:)
    CALL xios_orchidee_send_field("baresoilFrac",histvar)
    histvar(:)=SUM(frac_nobio(:,1:nnobio),dim=2)*100*contfrac(:)
    CALL xios_orchidee_send_field("residualFrac",histvar)

    CALL xios_orchidee_send_field("tsol_rad",tsol_rad-273.15)
    CALL xios_orchidee_send_field("qsurf",qsurf)
    CALL xios_orchidee_send_field("emis",emis)
    CALL xios_orchidee_send_field("z0m",z0m)
    CALL xios_orchidee_send_field("z0h",z0h)
    CALL xios_orchidee_send_field("roughheight",roughheight)

    ! calculate lai and laimean (pixel including no_bio)
    lai(:,ibare_sechiba) = zero
    histvar(:)=zero   
    DO ji = 1, kjpindex
       IF (SUM(veget_max(ji,:)) > zero) THEN
         DO jv=2,nvm
            lai(ji,jv) = cc_to_lai(circ_class_biomass(ji,jv,:,ileaf,icarbon),&
                 circ_class_n(ji,jv,:),jv)
            histvar(ji) = histvar(ji) + veget_max(ji,jv)*lai(ji,jv)/ &
                 SUM(veget_max(ji,:))
         END DO
       END IF
    END DO
    CALL xios_orchidee_send_field("lai",lai)
    CALL xios_orchidee_send_field("LAImean",histvar) 

    CALL xios_orchidee_send_field("vevapsno",vevapsno/dt_sechiba) 
    CALL xios_orchidee_send_field("vevapp",vevapp/dt_sechiba) 
    CALL xios_orchidee_send_field("vevapnu",vevapnu*one_day/dt_sechiba) 
    CALL xios_orchidee_send_field("vevapnu_alma",vevapnu/dt_sechiba) 
    CALL xios_orchidee_send_field("transpir",transpir*one_day/dt_sechiba) 
    CALL xios_orchidee_send_field("inter",vevapwet*one_day/dt_sechiba) 
    histvar(:)=zero 
    DO jv=1,nvm
      histvar(:) = histvar(:) + vevapwet(:,jv)
    ENDDO
    CALL xios_orchidee_send_field("ECanop",histvar/dt_sechiba)
    histvar(:)=zero
    DO jv=1,nvm
      histvar(:) = histvar(:) + transpir(:,jv)
    ENDDO
    CALL xios_orchidee_send_field("TVeg",histvar/dt_sechiba)
    CALL xios_orchidee_send_field("ACond",tq_cdrag)

    IF ( .NOT. almaoutput ) THEN
       ! Write history file in IPSL-format
       CALL histwrite_p(hist_id, 'beta', kjit, vbeta, kjpindex, index)
       CALL histwrite_p(hist_id, 'z0m', kjit, z0m, kjpindex, index)
       CALL histwrite_p(hist_id, 'z0h', kjit, z0h, kjpindex, index)
       CALL histwrite_p(hist_id, 'roughheight', kjit, roughheight, kjpindex, index)
       CALL histwrite_p(hist_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)

       ! calculate lai and laimean (pixel including no_bio fraction)
       lai(:,ibare_sechiba) = zero
       histvar(:)=zero   
       DO ji = 1, kjpindex
          IF (SUM(veget_max(ji,:)) > zero) THEN
             DO jv=2,nvm
                lai(ji,jv) = cc_to_lai(circ_class_biomass(ji,jv,:,ileaf,icarbon),&
                     circ_class_n(ji,jv,:),jv)
                histvar(ji) = histvar(ji) + veget_max(ji,jv)*lai(ji,jv)/ &
                     SUM(veget_max(ji,:))
             END DO
          END IF
       END DO
       CALL histwrite_p(hist_id, 'lai', kjit, lai, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'LAImean', kjit, histvar, kjpindex, index)

       CALL histwrite_p(hist_id, 'subli', kjit, vevapsno, kjpindex, index)
       CALL histwrite_p(hist_id, 'evapnu', kjit, vevapnu, kjpindex, index)
       CALL histwrite_p(hist_id, 'transpir', kjit, transpir, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'inter', kjit, vevapwet, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta1', kjit, vbeta1, kjpindex, index)
       CALL histwrite_p(hist_id, 'vbeta2', kjit, vbeta2, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta3', kjit, vbeta3, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta4', kjit, vbeta4, kjpindex, index)    
       CALL histwrite_p(hist_id, 'vbeta5', kjit, vbeta5, kjpindex, index)    
       CALL histwrite_p(hist_id, 'drysoil_frac', kjit, drysoil_frac, kjpindex, index)
       CALL histwrite_p(hist_id, 'rveget', kjit, rveget, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'rstruct', kjit, rstruct, kjpindex*nvm, indexveg)

       CALL histwrite_p(hist_id, 'snow', kjit, snow, kjpindex, index)
       CALL histwrite_p(hist_id, 'snowage', kjit, snow_age, kjpindex, index)
       CALL histwrite_p(hist_id, 'snownobio', kjit, snow_nobio, kjpindex*nnobio, indexnobio)
       CALL histwrite_p(hist_id, 'snownobioage', kjit, snow_nobio_age, kjpindex*nnobio, indexnobio)

       CALL histwrite_p(hist_id, 'grndflux', kjit, grndflux, kjpindex,index)
       CALL histwrite_p(hist_id, 'snowtemp',kjit,snowtemp,kjpindex*nsnow,indexsnow)
       CALL histwrite_p(hist_id, 'snowliq', kjit,snowliq,kjpindex*nsnow,indexsnow)
       CALL histwrite_p(hist_id, 'snowdz', kjit,snowdz,kjpindex*nsnow,indexsnow)
       CALL histwrite_p(hist_id, 'snowrho', kjit,snowrho,kjpindex*nsnow,indexsnow)
       CALL histwrite_p(hist_id, 'snowgrain',kjit,snowgrain,kjpindex*nsnow,indexsnow)
       CALL histwrite_p(hist_id, 'snowheat',kjit,snowheat,kjpindex*nsnow,indexsnow)
       
       CALL histwrite_p(hist_id,'frac_snow_veg',kjit,frac_snow_veg,kjpindex,index)
       CALL histwrite_p(hist_id, 'frac_snow_nobio', kjit,frac_snow_nobio,kjpindex*nnobio, indexnobio)
       CALL histwrite_p(hist_id, 'pgflux',kjit,pgflux,kjpindex,index)
       CALL histwrite_p(hist_id, 'soiltile',  kjit, soiltile, kjpindex*nstm, indexsoil)
       CALL histwrite_p(hist_id, 'soilindex',  kjit, REAL(njsc, r_std), kjpindex, index)
       CALL histwrite_p(hist_id, 'reinf_slope',  kjit, reinf_slope, kjpindex, index)
       CALL histwrite_p(hist_id, 'k_litt', kjit, k_litt, kjpindex, index)
       
       IF ( do_floodplains ) THEN
          CALL histwrite_p(hist_id, 'evapflo', kjit, vevapflo, kjpindex, index)
          CALL histwrite_p(hist_id, 'flood_frac', kjit, flood_frac, kjpindex, index)
       ENDIF
       CALL histwrite_p(hist_id, 'gsmean', kjit, gsmean, kjpindex*nvm, indexveg)    
       CALL histwrite_p(hist_id, 'gpp', kjit, gpp, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'cimean', kjit, cimean, kjpindex*nvm, indexveg)    
       IF ( ok_stomate ) THEN
          CALL histwrite_p(hist_id, 'nee', kjit, co2_flux, kjpindex*nvm, indexveg)    
       ENDIF
       IF ( ok_c13 ) THEN 
          CALL histwrite_p(hist_id, 'delta_c13_assim', kjit, delta_c13_assim, & 
          kjpindex*nvm, indexveg) 
          CALL histwrite_p(hist_id, 'leaf_ci_out', kjit, leaf_ci_out, kjpindex*nvm, & 
          indexveg) 
          CALL histwrite_p(hist_id, 'gpp_day', kjit, gpp_day, kjpindex*nvm, & 
          indexveg)             
       ENDIF

       histvar(:)=SUM(vevapwet(:,:),dim=2)
       CALL histwrite_p(hist_id, 'evspsblveg', kjit, histvar, kjpindex, index)

       histvar(:)= vevapnu(:)+vevapsno(:)
       CALL histwrite_p(hist_id, 'evspsblsoi', kjit, histvar, kjpindex, index)

       histvar(:)=SUM(transpir(:,:),dim=2)
       CALL histwrite_p(hist_id, 'tran', kjit, histvar, kjpindex, index)

       histvar(:)= sum_treefrac(:)*100*contfrac(:)
       CALL histwrite_p(hist_id, 'treeFrac', kjit, histvar, kjpindex, index) 

       histvar(:)= sum_grassfrac(:)*100*contfrac(:)
       CALL histwrite_p(hist_id, 'grassFrac', kjit, histvar, kjpindex, index) 

       histvar(:)= sum_cropfrac(:)*100*contfrac(:)
       CALL histwrite_p(hist_id, 'cropFrac', kjit, histvar, kjpindex, index)

       histvar(:)=veget_max(:,1)*100*contfrac(:)
       CALL histwrite_p(hist_id, 'baresoilFrac', kjit, histvar, kjpindex, index)

       histvar(:)=SUM(frac_nobio(:,1:nnobio),dim=2)*100*contfrac(:)
       CALL histwrite_p(hist_id, 'residualFrac', kjit, histvar, kjpindex, index)
    ELSE
       ! Write history file in ALMA format 
       CALL histwrite_p(hist_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
   
       ! calculate lai
       lai(:,ibare_sechiba) = zero  
       DO ji = 1, kjpindex
          DO jv=2,nvm
             lai(ji,jv) = cc_to_lai(circ_class_biomass(ji,jv,:,ileaf,icarbon),&
                  circ_class_n(ji,jv,:),jv)
          END DO
       END DO
       CALL histwrite_p(hist_id, 'lai', kjit, lai, kjpindex*nvm, indexveg)
       
       CALL histwrite_p(hist_id, 'ESoil', kjit, vevapnu, kjpindex, index)
       CALL histwrite_p(hist_id, 'EWater', kjit, vevapflo, kjpindex, index)
       CALL histwrite_p(hist_id, 'SWE', kjit, snow, kjpindex, index)
       histvar(:)=zero
       DO jv=1,nvm
          histvar(:) = histvar(:) + transpir(:,jv)
       ENDDO
       CALL histwrite_p(hist_id, 'TVeg', kjit, histvar, kjpindex, index)
       histvar(:)=zero
       DO jv=1,nvm
          histvar(:) = histvar(:) + vevapwet(:,jv)
       ENDDO
       CALL histwrite_p(hist_id, 'ECanop', kjit, histvar, kjpindex, index)
       CALL histwrite_p(hist_id, 'ACond', kjit, tq_cdrag, kjpindex, index)
       CALL histwrite_p(hist_id, 'SnowFrac', kjit, vbeta1, kjpindex, index)
       !
       CALL histwrite_p(hist_id, 'Z0m', kjit, z0m, kjpindex, index)
       CALL histwrite_p(hist_id, 'Z0h', kjit, z0h, kjpindex, index)
       CALL histwrite_p(hist_id, 'EffectHeight', kjit, roughheight, kjpindex, index)
       !
       IF ( do_floodplains ) THEN
          CALL histwrite_p(hist_id, 'Qflood', kjit, vevapflo, kjpindex, index)
          CALL histwrite_p(hist_id, 'FloodFrac', kjit, flood_frac, kjpindex, index)
       ENDIF
       !
       CALL histwrite_p(hist_id, 'gsmean', kjit, gsmean, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'cimean', kjit, cimean, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'GPP', kjit, gpp, kjpindex*nvm, indexveg)
       
       IF ( ok_stomate ) THEN
             CALL histwrite_p(hist_id, 'NEE', kjit, co2_flux, kjpindex*nvm, indexveg)    
       ENDIF
    ENDIF ! almaoutput
    
    !! 13. Write additional output file with higher frequency
    IF ( hist2_id > 0 ) THEN
       IF ( .NOT. almaoutput ) THEN
          WRITE(numout,*)'vbeta, what is happening',vbeta
          ! Write history file in IPSL-format
          CALL histwrite_p(hist2_id, 'tsol_rad', kjit, tsol_rad, kjpindex, index)
          CALL histwrite_p(hist2_id, 'qsurf', kjit, qsurf, kjpindex, index)
          CALL histwrite_p(hist2_id, 'albedo', kjit, albedo, kjpindex*2, indexalb)
          CALL histwrite_p(hist2_id, 'emis', kjit, emis, kjpindex, index)
          CALL histwrite_p(hist2_id, 'beta', kjit, vbeta, kjpindex, index)
          CALL histwrite_p(hist2_id, 'z0m', kjit, z0m, kjpindex, index)
          CALL histwrite_p(hist2_id, 'z0h', kjit, z0h, kjpindex, index)
          CALL histwrite_p(hist2_id, 'roughheight', kjit, roughheight, kjpindex, index)
          CALL histwrite_p(hist2_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)

          ! calculate lai
          lai(:,ibare_sechiba) = zero  
          DO ji = 1, kjpindex
             DO jv=2,nvm
                lai(ji,jv) = cc_to_lai(circ_class_biomass(ji,jv,:,ileaf,icarbon),&
                     circ_class_n(ji,jv,:),jv)
             END DO
          END DO
          CALL histwrite_p(hist2_id, 'lai', kjit, lai, kjpindex*nvm, indexveg)
          
          CALL histwrite_p(hist2_id, 'subli', kjit, vevapsno, kjpindex, index)
          IF ( do_floodplains ) THEN
             CALL histwrite_p(hist2_id, 'vevapflo', kjit, vevapflo, kjpindex, index)
             CALL histwrite_p(hist2_id, 'flood_frac', kjit, flood_frac, kjpindex, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'vevapnu', kjit, vevapnu, kjpindex, index)
          CALL histwrite_p(hist2_id, 'transpir', kjit, transpir, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'inter', kjit, vevapwet, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta1', kjit, vbeta1, kjpindex, index)
          CALL histwrite_p(hist2_id, 'vbeta2', kjit, vbeta2, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta3', kjit, vbeta3, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta4', kjit, vbeta4, kjpindex, index)    
          CALL histwrite_p(hist2_id, 'vbeta5', kjit, vbeta5, kjpindex, index)    
          CALL histwrite_p(hist2_id, 'drysoil_frac', kjit, drysoil_frac, kjpindex, index)
          CALL histwrite_p(hist2_id, 'rveget', kjit, rveget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'rstruct', kjit, rstruct, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'snow', kjit, snow, kjpindex, index)
          CALL histwrite_p(hist2_id, 'snowage', kjit, snow_age, kjpindex, index)
          CALL histwrite_p(hist2_id, 'snownobio', kjit, snow_nobio, kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist2_id, 'snownobioage', kjit, snow_nobio_age, kjpindex*nnobio, indexnobio)

          CALL histwrite_p(hist2_id, 'soilindex',  kjit, REAL(njsc, r_std), kjpindex, index)
          CALL histwrite_p(hist2_id, 'reinf_slope',  kjit, reinf_slope, kjpindex, index)
          
          IF (ok_hydrol_arch) THEN
              CALL histwrite_p(hist2_id, 'transpir_supply', kjit,transpir_supply, kjpindex*nvm, indexveg)
          ENDIF

          CALL histwrite_p(hist2_id, 'gsmean', kjit, gsmean, kjpindex*nvm, indexveg)    
          CALL histwrite_p(hist2_id, 'gpp', kjit, gpp, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'cimean', kjit, cimean, kjpindex*nvm, indexveg)    
          
          IF ( ok_stomate ) THEN
             CALL histwrite_p(hist2_id, 'nee', kjit, co2_flux, kjpindex*nvm, indexveg)    
          ENDIF
       ELSE
          ! Write history file in ALMA format
          CALL histwrite_p(hist2_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist2_id, 'ESoil', kjit, vevapnu, kjpindex, index)
          IF ( do_floodplains ) THEN
             CALL histwrite_p(hist2_id, 'EWater', kjit, vevapflo, kjpindex, index)
             CALL histwrite_p(hist2_id, 'FloodFrac', kjit, flood_frac, kjpindex, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'SWE', kjit, snow, kjpindex, index)
          histvar(:)=zero
          DO jv=1,nvm
             histvar(:) = histvar(:) + transpir(:,jv)
          ENDDO
          CALL histwrite_p(hist2_id, 'TVeg', kjit, histvar, kjpindex, index)
          histvar(:)=zero
          DO jv=1,nvm
             histvar(:) = histvar(:) + vevapwet(:,jv)
          ENDDO
          CALL histwrite_p(hist2_id, 'ECanop', kjit, histvar, kjpindex, index)
          CALL histwrite_p(hist2_id, 'ACond', kjit, tq_cdrag, kjpindex, index)
          CALL histwrite_p(hist2_id, 'SnowFrac', kjit, vbeta1, kjpindex, index)
          CALL histwrite_p(hist2_id, 'GPP', kjit, gpp, kjpindex*nvm, indexveg)
          
          IF ( ok_stomate ) THEN
             CALL histwrite_p(hist2_id, 'NEE', kjit, co2_flux, kjpindex*nvm, indexveg)    
          ENDIF
       ENDIF ! almaoutput
    ENDIF ! hist2_id

    !#481
    !WRITE(numout,*) 'AHAAA: before slowproc_change_frac veget_max', veget_max(:,1)

    !! Change the vegetation fractions if a new map was read in slowproc. 
    !  This is done after lcchange has been done in stomatelpj
    IF (done_stomate_lcchange) THEN
       CALL slowproc_change_frac(kjpindex, veget_max, veget, &
            frac_nobio, totfrac_nobio, tot_bare_soil, soiltile, &
            circ_class_biomass, circ_class_n, lai_per_level,&
            z_array_out)
       done_stomate_lcchange=.FALSE.
    END IF

    !#481
    !WRITE(numout,*) 'AHAAA: before sechiba_finalize veget_max', veget_max(:,1)
    
    !! 14. If it is the last time step, write restart files
    IF (ldrestart_write) THEN
       CALL sechiba_finalize( &
            kjit,     kjpij,  kjpindex, index,   rest_id, &
            tq_cdrag, vevapp, fluxsens, fluxlat, tsol_rad, &
            veget_max)
    END IF

  END SUBROUTINE sechiba_main


!!  =============================================================================================================================
!! SUBROUTINE:    sechiba_mleb_hs
!!
!>\BRIEF:	      call the relevant modules for the multi-layer energy budget and hydraulic stress
!!
!! DESCRIPTION:	  call the relevant modules for the multi-layer energy budget and hydraulic stress
!!
!! \n
!_ ==============================================================================================================================

  SUBROUTINE sechiba_mleb_hs( &
       kjit,         kjpij,        kjpindex,     index,                   &
       lalo,         contfrac,     neighbours,   resolution, zlev,        &
       u,            v,            qair,         t2m,        temp_air,    &
       petAcoef,     peqAcoef,     petBcoef,     peqBcoef,                &
       precip_rain,  precip_snow,  lwdown,       swnet,      swdown,      &
       pb,           rest_id,      hist_id,      hist2_id,                &
       rest_id_stom, hist_id_stom, hist_id_stom_IPCC,                     &
       fluxsens,     fluxlat,      temp_sol_new, tsol_rad,   vevapp,      &
       vbeta2sum,    vbeta3sum,    tq_cdrag,     ldrestart_read,          &
       ldrestart_write, epot_air,  flux_ground_h, flux_ground_le,         &
       temp_surf_pres, temp_surf_next, sinang, q2m, ccanopy, vbeta23,     &
       leaf_ci, gs_distribution, gs_diffuco_output, gstot_component,      &
       gstot_frac)


!! 0.1 Input variables    

!! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                               :: kjit              !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpij             !! Total size of the un-compressed grid 
                                                                                  !! (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpindex          !! Domain size - terrestrial pixels only 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id           !! _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id           !! _History_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist2_id          !! _History_ file 2 identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id_stom      !! STOMATE's _Restart_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id_stom      !! STOMATE's _History_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT(in)                                :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file file 
                                                                                  !! identifier (unitless)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)          :: lalo              !! Geographic coordinates (latitude,longitude)
                                                                                  !! for grid cells (degrees)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: contfrac          !! Fraction of continent in the grid 
                                                                                  !! (unitless, 0-1)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index             !! Indices of the pixels on the map. 
                                                                                  !! Sechiba uses a reduced grid excluding oceans
                                                                                  !! ::index contains the indices of the 
                                                                                  !! terrestrial pixels only! (unitless)

    LOGICAL, INTENT(in)                                      :: ldrestart_read    !! Logical for _restart_ file to read 
                                                                                  !! (true/false)
    LOGICAL, INTENT(in)                                      :: ldrestart_write   !! Logical for _restart_ file to write 
    INTEGER(i_std), DIMENSION (kjpindex,8), INTENT(in)       :: neighbours        !! Neighboring grid points if land!(unitless)
    REAL(r_std), DIMENSION (kjpindex,2), INTENT(in)          :: resolution        !! Size in x and y of the grid (m)
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u                 !! Lowest level wind speed in direction u 
                                                                                  !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: v                 !! Lowest level wind speed in direction v 
                                                                                  !! @tex $(m.s^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev              !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair              !! Lowest level specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: t2m               !! 2m air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_rain       !! Rain precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_snow       !! Snow precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown            !! Down-welling long-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet             !! Net surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swdown            !! Down-welling surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_air          !! Air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef          !! Coefficients A for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef          !! Coefficients A for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef          !! Coefficients B for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef          !! Coefficients B for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb                !! Surface pressure (hPa)

    REAL(r_std),DIMENSION (kjpindex)                         :: tq_cdrag          !! Surface drag coefficient (-) 
                                                                                  !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex)                         :: temp_sol_new      !! New ground temperature (K)

    REAL(r_std),DIMENSION (kjpindex)                         :: tsol_rad          !! Radiative surface temperature 
                                                                                  !! @tex $(W m^{-2})$ @endtex 

    REAL(r_std), DIMENSION(kjpindex)                         :: vbeta2sum         !! sum of vbeta2 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION(kjpindex)                         :: vbeta3sum         !! sum of vbeta3 coefficients across all PFTs (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air          !! Air potential energy (??J)

    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: sinang            !! Sine of the solar angle (unitless)

    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q2m               !! 2m specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex

    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: ccanopy           !! CO2 concentration in the canopy (ppm)

    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)        :: vbeta23           !! Beta for fraction of wetted foliage that will
                                                                                  !! transpire once intercepted water has evaporated (-)
    REAL(r_std), DIMENSION (kjpindex,nvm,nlai), INTENT(in)   :: leaf_ci           !! intercellular CO2 concentration (ppm)

    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in)       :: gs_distribution
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in)       :: gs_diffuco_output
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in)       :: gstot_component
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in)       :: gstot_frac

    REAL(r_std)                                              :: flux_ground_h     !! sensible heat flux at surface (W^{m^2})
    REAL(r_std)                                              :: flux_ground_le    !! latent heat flux at surface (W^{m^2})

    REAL(r_std)                                              :: temp_surf_pres    !! temperature at surface for the present timestep (K)
    REAL(r_std)                                              :: temp_surf_next    !! temperature at surface for the next timestep (K)
    INTEGER(i_std)                                           :: james_step_count  !! james_step_count

    REAL(r_std),DIMENSION (kjpindex)                         :: netrad_out        !! Net radiation flux add by YC 20140313

!! 0.2 Output variables

!! 0.3 Modified variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: fluxsens          !! Sensible heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: fluxlat           !! Latent heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: vevapp            !! Total of evaporation 
                                                                                  !! @tex $(kg m^{-2} days^{-1})$ @endtex

!! 0.4 Local variables
    ! variables introduced as a consequence of running enerbil twice (if needed, as a result of restriction of transpiration by
    !    the supply term). If running for the second time, we should start with the same input variables as the first run
    
    INTEGER(i_std)                           :: ji, jv
    REAL(r_std), DIMENSION (kjpindex)        :: evapot_save           !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex)        :: evapot_corr_save      !! Soil Potential Evaporation Correction (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex)        :: temp_sol_save         !! Soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex)        :: qsurf_save            !! Surface specific humidity (kg kg^{-1})     
    REAL(r_std), DIMENSION (kjpindex)        :: fluxsens_save         !! Sensible heat flux (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: fluxlat_save          !! Latent heat flux (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: tsol_rad_save         !! Tsol_rad (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: vevapp_save           !! Total of evaporation (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: gpp_save              !! Assimilation ((gC m^{-2}), total area)     
    REAL(r_std), DIMENSION (kjpindex)        :: temp_sol_new_save     !! New soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta2sum_save        !! sum of vbeta2 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta3sum_save        !! sum of vbeta3 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3_save           !! Beta for Transpiration (unitless)
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3pot_save        !! Beta for Potential Transpiration
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: rveget_save           !! stomatal resistance of vegetation 
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: rstruct_save          !! structural resistance @tex ($s m^{-1}$) @endtex
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: cimean_save           !! mean intercellular CO2 concentration
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: gsmean_save           !! mean stomatal conductance to CO2 (umol m-2 s-1)
   ! REAL(r_std), DIMENSION (kjpindex,nvm)   :: gpp_save
    REAL(r_std), DIMENSION (kjpindex)        :: qsol_sat_new_out      !! New saturated surface air moisture (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex)         :: qair_new
    REAL(r_std),DIMENSION (kjpindex)         :: delta_temp_save
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3_oldvalue
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta3_save_test
    
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: transpir_init         !! Temperate storage variable for the transpiration before limited by transpiration supply  
  !  REAL(r_std),DIMENSION(kjpindex,nvm,nlevels_tot)    :: z_array3      !! Same as z_array, but one less dimension.
  !                                                                     !! @tex $(m)$ @endte

    REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm)        :: laieff_collim          !! (FROM STOMATE) Leaf Area Index Effective for direct light

  !  REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot)   :: h_array_out             !! (FROM STOMATE) An output of h_array, to use in sechiba               
  !  REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot)   :: z_array_out             !! (FROM STOMATE) An output of h_array, to use in sechiba                                                                           

    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_vbeta3
    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_rveget

    REAL(r_std), DIMENSION(nlevels_tot, kjpindex, nvm)      :: transpir_supply_column   !! (SECHIBA TOP LEVEL) Supply of water for transpiration 
 !  REAL(r_std), DIMENSION (kjpindex,nvm)                   :: max_height_store     

    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Abs_Tot           !! (SECHIBA TOP LEVEL) collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Alb_Tot           !! (SECHIBA TOP LEVEL) Collimated (direct) total albedo for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: laieff_isotrop_pft       !! (FROM STOMATE) collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: laieff_collim_pft        !! (FROM STOMATE) Collimated (direct) total albedo for a given level

    ! %%% begin variables for the enerbil and hydrol interface %%%%

    LOGICAL                                                 :: hydrol_flag           !! flag that 'trips' the energy budget for each grid square 
    LOGICAL, DIMENSION (kjpindex)                           :: hydrol_flag2          !! flag that 'trips' the energy budget for each grid square
    LOGICAL, DIMENSION (kjpindex, nvm)                      :: hydrol_flag3          !! flag that 'trips' the energy budget for each grid square and PFT

    LOGICAL, DIMENSION (kjpindex, nlevels_tot)              :: hydrol_flag2_column   !! flag that 'trips' the energy budget for each grid square, and canopy level
    LOGICAL, DIMENSION (kjpindex, nvm, nlevels_tot)         :: hydrol_flag3_column   !! flag that 'trips' the energy budget for each grid square, canopy level and PFT


    ! %%% end variables for the enerbil and hydrol interface %%%%

! ==============================================================================================================================


        ! we define here a set of flags, required to signal if we need to recalculate the
        ! energy budget depending on the hydraulic stress being a limiting factor to the
        ! latent heat flux.
       
        hydrol_flag = .FALSE.     ! trip flag per cycle (no dimension)
        hydrol_flag2(:) = .FALSE.    ! trip flag per grid square
        hydrol_flag3(:,:) = .FALSE.    ! trip flag per grid square and PFT
       
        ! ---------------------------------------------------------------------------------------------------
        ! before first calling diffuco_trans_co2 again, store output values from the first call to diffuco
   
        vbeta3_save(:,:) = vbeta3(:,:)           !! Beta for Transpiration (unitless)
        vbeta3pot_save(:,:) = vbeta3pot(:,:)     !! Beta for Potential Transpiration
        rveget_save(:,:) = rveget(:,:)           !! stomatal resistance of vegetation 
        rstruct_save(:,:) = rstruct(:,:)         !! structural resistance @tex ($s m^{-1}$) @endtex
        cimean_save(:,:) = cimean(:,:)           ! mean intercellular CO2 concentration
        gsmean_save(:,:) = gsmean(:,:)           ! mean stomatal conductance to CO2 (umol m-2 s-1)
        gpp_save(:,:) = gpp(:,:)                 !! gross primary production
        ! ---------------------------------------------------------------------------------------------------
       
        ! ---------------------------------------------------------------------------------------------------
        ! before first call to enerbil, store initial values
  
        evapot_save(:) = evapot(:)                !! Soil Potential Evaporation (mm day^{-1})
        evapot_corr_save(:) = evapot_corr(:)      !! Soil Potential Evaporation Correction (mm day^{-1})
        qsurf_save(:) = qsurf(:)                  !! Surface specific humidity (kg kg^{-1})

        fluxsens_save(:) = fluxsens(:)            !! Sensible heat flux (W m^{-2})
        fluxlat_save(:) = fluxlat(:)              !! Latent heat flux (W m^{-2})

        temp_sol_save(:) = temp_sol(:)            !! Soil temperature (K)
        temp_sol_new_save(:) = temp_sol_new(:)    !! New soil temperature (K)
        tsol_rad_save(:) = tsol_rad(:)            !! Tsol_rad (W m^{-2})
        vevapp_save(:) = vevapp(:)                !! Total of evaporation (mm day^{-1})
        gpp_save(:,:) = gpp(:,:)                  !! Assimilation ((gC m^{-2}), total area)
        vbeta2sum_save(:) = vbeta2sum(:) 
        vbeta3sum_save(:) = vbeta3sum(:)
        ! ---------------------------------------------------------------------------------------------------

        IF(printlev_loc>=4) WRITE(numout, *) '081116, in sechiba_mleb_hs epot_air',epot_air 
        IF(printlev_loc>=3) WRITE(numout, *) '140616 about to call mleb_main'
        CALL mleb_main (kjit, kjpindex, ldrestart_read, ldrestart_write, &
             & index, indexveg, zlev, lwdown, swnet, swdown, epot_air, temp_air, &
             & u, v, petAcoef, petBcoef, qair, peqAcoef, peqBcoef, pb, rau, &
             & vbeta, vbeta1, vbeta2, vbeta3, vbeta3pot, vbeta4, vbeta5, &
             & emis, soilflx, soilcap, tq_cdrag, humrel, &
             & fluxsens, fluxlat, vevapp, transpir, transpot, vevapnu, vevapwet, &
             & vevapsno, vevapflo, temp_sol, tsol_rad, temp_sol_new, qsurf, &
             & evapot, evapot_corr, rest_id, hist_id, hist2_id, & 
             & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, t_a_next_grid, &
             & q_a_next_grid, temp_surf_next, temp_atmos_pres_grid, q_atmos_pres_grid, &
             & ok_mleb_history_file, james_step_count, temp_leaf_pres_grid, temp_surf_pres, &
             & transpir_supply, hydrol_flag, hydrol_flag2, hydrol_flag3, vbeta2sum, & 
             & vbeta3sum, veget_max, qsol_sat_new_out, qair_new, delta_temp_save, &
             & netrad_out, veget, Collim_Abs_Tot, Collim_Alb_Tot, laieff_isotrop, &
             & h_array_out, z_array_out, transpir_supply_column, u_speed, &
             & profile_vbeta3, profile_rveget, max_height_store, &
             & u_speed_grid, flux_rn_grid, t2m_month_out) 

        delta_temp_save = temp_surf_next - temp_sol
        ! these are calculated in the first call to the mleb_main, but may be required if it is called again
        fluxsens_save(:) = fluxsens(:)            !! Sensible heat flux (W m^{-2})
        fluxlat_save(:) = fluxlat(:)              !! Latent heat flux (W m^{-2})

       
        !+++CHECK+++
        ! Store the proxy for unstressed ecosystem functioning. This could be gpp, transpiration,
        ! evaporation. They all differ conceptually and numerically but the idea is the same.
        ! In the initial approach we tried to calculate the stress as a ratio of the initial 
        ! transpiration over water supplied through the plant.  This approach resulted
        ! in inconsistencies likely because the many feedbacks in the calculation of transpir. 
        ! Similar issues were observed when using transport. For this reason it was decided to drop
        ! this approach and use the ratio between the initial and adjusted gpp instead. After
        ! all this ratio will be used in stomate to adjust the allocation.
        ! Note that the feedback on gpp is calculated based on the ratio between ::transpir_supply 
        ! and transpir at the half hourly time step. In stomate we use the ratio between stressed
        ! and unstressed but at the daily time step. Night time values should be ignored in the
        ! daily calculation. If night time values are considered to unstressed this will skew
        ! the result.
        unstressed(:,:) = gpp(:,:)
        !+++++++++++
        IF (ok_hydrol_arch) THEN

           CALL hydrol_arch (kjpindex, circ_class_biomass, circ_class_n, temp_sol_new, &
                temp_air, swc, transpir_supply, njsc, soiltile, &
                ! vir_transpir_supply, veget, h_array_out, h_array_out, z_array_out3, &
                vir_transpir_supply, veget, h_array_out, z_array_out, &
                transpir_supply_column,ksave,e_frac)

           ! transpir_supply is provided in mm per timestep (dt_sechiba)
           transpir_init(:,:) = zero
 
           DO ji = 1, kjpindex

              !don't restrict gs during night
              DO jv = 1, nvm
                 IF (sinang(ji) .LT. min_sechiba) THEN
                    ! Night time we will never recalculate the energy budget
                    hydrol_flag2(ji) = .FALSE.
                    hydrol_flag3(ji,jv) = .FALSE.
                    transpir_init(ji, jv)= zero
                 ELSE
                   ! The ratio between ::transpir_supply and ::transpir is used to determine
                    ! whether we will recalculate gpp, transpir, ... (thus enerbil). This
                    ! is done every half hour. The waterstress that will be passed to stomate
                    ! to adjust the allocation will be calculated at the daily time step.
                    ! It is important NOT to include night time values in the daily mean
                    ! because more southern sides have shorter days during the growing season
                    ! the number of half hours with day light biases the waterstress if it 
                    ! is assumed that there is no waterstress at night.

                    IF (veget_max(ji,jv) .GT. zero .AND. &
                         veget(ji,jv) .GT. zero) THEN  
                        transpir_init(ji, jv)= (transpir(ji,jv)/veget_max(ji,jv)) 
                       !comparing the transpiration in tree-stand scale   
                       IF ( transpir_supply(ji,jv) .LT. transpir_init(ji,jv) ) THEN 
                          IF(printlev_loc>=3) WRITE(numout,*) '091013 hydrol_flag2&3 set to TRUE, pft is: ', jv
                          hydrol_flag2(ji) = .TRUE.
                          hydrol_flag3(ji,jv) = .TRUE.
                       ELSE
                          IF(printlev_loc>=3) WRITE(numout,*) '091013 hydrol_flag2 set to FALSE, pft is: ', jv
                       END IF
                    ELSE
                       ! There is no vegetation here, so we can skip it.
                       ! the flag at the pixel level (::hydrol_flag2) should 
                       ! not change its value.
                       hydrol_flag3(ji,jv) = .FALSE.
                       transpir_init(ji, jv)= zero
                    ENDIF
                    
                    !---TEMP---
                    IF((printlev_loc>=3) .AND. (jv.EQ.test_pft)) THEN
                       WRITE(numout,*) 'YC transpir_init:',   transpir_init(ji, jv)
                       WRITE(numout,*) 'transpir_supply, ',transpir_supply(ji,test_pft)
                       IF (veget_max(ji,test_pft) .GT. zero) THEN  
                          WRITE(numout,*) 'transpir modified, ', transpir(ji,test_pft)/veget_max(ji,test_pft) 
                       ENDIF
                    ENDIF
                    !----------

                 END IF

              END DO ! jv = 1, nvm

           END DO ! ji = 1, kjpindex
           ! The energy budget is only calculated again, if the above conditions are satisfied 
           ! for any of the grid squares, and the feedback of waterstress on stomatal 
           ! closure (ok_gs_feedback) is activated
           
           DO ji = 1, kjpindex  ! grid point cycle

              IF  (hydrol_flag2(ji)) THEN    ! this switch is 'tripped' if the above applies to any PFT at the
                 ! grid point, as the whole energy budget for that grid point must
                 ! then be recalculated, and mleb_main thus needs to be called
                 ! again
                 hydrol_flag = .TRUE.
              ELSE
                 ! move on to the next grid point in question
              END IF
              
           END DO ! ji = 1, kjpindex
        
           ! If the feedback of water stress on stomatal closure is not
           ! activated, then hydrol_flag is set to false, and no energy
           ! re-calculations are conducted

           IF (.NOT. ok_gs_feedback) hydrol_flag=.FALSE.

           IF (hydrol_flag) THEN
           
              ! mleb_main should be called here again (to recalculate the energy budget because of
              !     supply term constriction
           
              ! ---------------------------------------------------------------------------------------------------  
              ! before 2nd call to enerbil, recall initial values (the ones un-changed from the first enerbil call)
           
              evapot(:) = evapot_save(:)                   !! Soil Potential Evaporation (mm day^{-1})
              evapot_corr(:) = evapot_corr_save(:)         !! Soil Potential Evaporation Correction (mm day^{-1})
              temp_sol(:) = temp_sol_save(:)               !! Soil temperature (K)
              qsurf(:) = qsurf_save(:)                     !! Surface specific humidity (kg kg^{-1})
              fluxsens(:) = fluxsens_save(:)               !! Sensible heat flux (W m^{-2})
              fluxlat(:) = fluxlat_save(:)                 !! Latent heat flux (W m^{-2})
              tsol_rad(:) = tsol_rad_save(:)               !! Tsol_rad (W m^{-2})
              vevapp(:) = vevapp_save(:)                   !! Total of evaporation (mm day^{-1})
              gpp(:,:) = gpp_save(:,:)                     !! Assimilation ((gC m^{-2}), total area)
              temp_sol_new(:) = temp_sol_new_save(:)       !! New soil temperature (K)
              vbeta2sum(:) = vbeta2sum_save(:) 
              vbeta3sum(:) = vbeta3sum_save(:)   
              ! ---------------------------------------------------------------------------------------------------
              DO ji = 1, kjpindex  ! grid point cycle
                 
                 ! 201013 calculate new series of beta_v3
                    vbeta3_oldvalue(:, :) = 0.0d0
                 
                    ! vbeta2sum(:) = zero
                    IF (hydrol_flag2(ji)) THEN
                       DO jv = 1, nvm
                          IF (.NOT. hydrol_flag3(ji, jv)) THEN 
                             ! this is for cases when the supply term is higher than the 
                             !     transpiration) - i.e. the normal state of affairs
                          ELSE
                             vbeta3_oldvalue(ji, jv) = vbeta3(ji, jv)

                             ! We calculate the vbeta3 based on the ::transpir_supply term for each grid and pft
                             ! We should keep the spatial weighting fraction,::veget_max, for vebta3. Even the calculation
                             ! of ::transpir_supply is in tree stand scale.   
                             IF (veget_max(ji,jv) .GT. zero .AND. veget(ji,jv) .GT. zero) THEN        
                                  vbeta3(ji,jv) = veget_max(ji, jv) * transpir_supply(ji,jv)  / &  
                                       &  ( dt_sechiba * (1.0d0 - vbeta1(ji)) * (qsol_sat_new_out(ji) - qair_new(ji)) * &
                                       &  ( rau(ji) * (MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))) * tq_cdrag(ji)))
                             ELSE
                                  ! No vegetation present for this condition, so vbeta3 is set equal to zero. 
                                  vbeta3(ji,jv) = 0.0d0
                             ENDIF
 
                              IF ( ( vbeta3(ji, jv)/vbeta3_oldvalue(ji, jv) ) .gt. 1.0d0 ) THEN
                                ! WRITE(numout,*) 'vbeta3 ratio is greater than 1!!'
                              END IF
                          END IF ! (hydrol_flag3(ji, jv) .eq. .FALSE.)
                       ENDDO ! jv = 1, nvm
                    ELSE
                       ! do nothing
                    END IF ! (hydrol_flag2(ji) .eq. .TRUE.)        
              END DO ! ji = 1, kjpindex
              
              ! now to approximate the stomatatal conductance, where it is constrained by the supply term.
              ! This calculation takes place over the whole canopy, for now.

              ! To calculate the gs_effective (the effective stomatal conductance), first we need to calculate
              !   a constrained vbeta3, and work back from there, using a tweaked version of diffuco_trans_co2

              ! IF (ld_mleb_write) THEN
                  ! write statements for error checking
              !     WRITE(numout, *) 'before mleb_main'
              ! END IF !(ld_mleb_write)

              CALL mleb_main (kjit, kjpindex, ldrestart_read, ldrestart_write, &
                   & index, indexveg, zlev, lwdown, swnet, swdown, epot_air, temp_air, &
                   & u, v, petAcoef, petBcoef, qair, peqAcoef, peqBcoef, pb, rau, &
                   & vbeta, vbeta1, vbeta2, vbeta3, vbeta3pot, vbeta4, vbeta5, &
                   & emis, soilflx, soilcap, tq_cdrag, humrel, &
                   & fluxsens, fluxlat, vevapp, transpir, transpot, vevapnu, vevapwet, &
                   & vevapsno, vevapflo, temp_sol, tsol_rad, temp_sol_new, qsurf, &
                   & evapot, evapot_corr, rest_id, hist_id, hist2_id, & 
                   & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, t_a_next_grid, q_a_next_grid, & 
                   & temp_surf_next, temp_atmos_pres_grid, q_atmos_pres_grid, &
                   & ok_mleb_history_file, james_step_count, temp_leaf_pres_grid, temp_surf_pres, &
                   & transpir_supply, hydrol_flag, hydrol_flag2, &
                   & hydrol_flag3, vbeta2sum, vbeta3sum, veget_max, qsol_sat_new_out, qair_new, &
                   & delta_temp_save, netrad_out, veget, Collim_Abs_Tot, Collim_Alb_Tot, &
                   & laieff_isotrop, h_array_out, z_array_out, transpir_supply_column, &
                   & u_speed, profile_vbeta3, profile_rveget, max_height_store, &
                   & u_speed_grid, flux_rn_grid, t2m_month_out )
              
              ! we need to pass on the new vbeta3 to mleb_diffuco_trans_co2

              CALL mleb_diffuco_trans_co2 (kjpindex, swdown, temp_sol, pb, qsurf, &
                   & q2m, t2m, temp_growth, rau, u, v, tq_cdrag, vegstress, &
                   & assim_param, ccanopy, veget_max, qsintveg, qsintmax, &
                   & vbeta3, vbeta3pot, rveget, rstruct, cimean, gsmean, gpp, &
                   & vbeta23, Isotrop_Abs_Tot_p, Isotrop_Tran_Tot_p, lai_per_level, &
                   & leaf_ci, hydrol_flag2, hydrol_flag3, gs_distribution, gs_diffuco_output, &
                   & ok_mleb_history_file, gstot_component, gstot_frac, warnings, veget, &
                   & circ_class_biomass,circ_class_n, u_speed, profile_vbeta3, profile_rveget)
 
              DO ji = 1, kjpindex

                 DO  jv = 1, nvm

                    IF (hydrol_flag3(ji,jv)) THEN

                       ! do nothing (as the relevant values were calculated in mleb_diffuco_trans_co2)
                       
                    ELSE
                       ! ------------------------------------------------------------------------------------------
                       ! restore saved values for PFTs and grid squares not affected by water stress
                       vbeta3(ji,jv) = vbeta3_save(ji,jv)        !! Beta for Transpiration (unitless)
                       vbeta3pot(ji,jv) = vbeta3pot_save(ji,jv)  !! Beta for Potential Transpiration
                       rveget(ji,jv) = rveget_save(ji,jv)        !! stomatal resistance of vegetation 
                       rstruct(ji,jv) = rstruct_save(ji,jv)      !! structural resistance @tex ($s m^{-1}$) @endtex
                       cimean(ji,jv) = cimean_save(ji,jv)        ! mean intercellular CO2 concentration
                       gsmean(ji,jv) = gsmean_save(ji,jv)        ! mean stomatal conductance to CO2 (umol m-2 s-1)
                       gpp(ji,jv) = gpp_save(ji,jv)              !! gross primary production
                       ! ------------------------------------------------------------------------------------------

                    END IF

                 END DO ! jv = 1, nvm
                 
              END DO ! ji = 1, kjpindex

           ELSE
              ! do nothing
           END IF ! (hydrol_flag .eq. .TRUE.) 

           !+++CHECK+++
           ! Store the proxy for stressed ecosystem functioning. This value is used in stomate
           ! to calculate the waterstress used in allocation
           stressed(:,:) = gpp(:,:)
           !+++++++++++
                
        END IF   ! IF (ok_hydrol_arch) THEN

        ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        mleb_count = mleb_count + 1

        !! 2.3.1 Write enerbil data to the history files.
!!$        CALL enerbil_write (kjit, kjpindex, index, lwdown, ccanopy, temp_sol, &
        CALL enerbil_write (kjit, kjpindex, index, lwdown, temp_sol, &
            & temp_sol_new, evapot, evapot_corr, hist_id, hist2_id)


    
  END SUBROUTINE sechiba_mleb_hs


!!  =============================================================================================================================
!! SUBROUTINE:    sechiba_finalize
!!
!>\BRIEF	  Finalize all modules by calling their "_finalize" subroutines.
!!
!! DESCRIPTION:	  Finalize all modules by calling their "_finalize" subroutines. These subroutines will write variables to 
!!                restart file. 
!!
!! \n
!_ ==============================================================================================================================

  SUBROUTINE sechiba_finalize( &
       kjit,     kjpij,  kjpindex, index,   rest_id, &
       tq_cdrag, vevapp, fluxsens, fluxlat, tsol_rad, &
       veget_max)

!! 0.1 Input variables    
    INTEGER(i_std), INTENT(in)                               :: kjit              !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpij             !! Total size of the un-compressed grid 
                                                                                  !! (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpindex          !! Domain size - terrestrial pixels only 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id           !! _Restart_ file identifier (unitless)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index             !! Indices of the pixels on the map. 
                                                                                  !! Sechiba uses a reduced grid excluding oceans
                                                                                  !! ::index contains the indices of the 
                                                                                  !! terrestrial pixels only! (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: tsol_rad           !! Radiative surface temperature 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: vevapp             !! Total of evaporation 
                                                                                  !! @tex $(kg m^{-2} days^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: fluxsens           !! Sensible heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: fluxlat            !! Latent heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: tq_cdrag           !! Surface drag coefficient (-)
    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)       :: veget_max          !! Max. fraction of vegetation type (inc. no_bio fraction)


!! 0.2 Local variables
    INTEGER(i_std)                                          :: ji, jv		  !! Index (unitless)
    REAL(r_std), DIMENSION(kjpindex)                        :: histvar            !! Computations for history files (unitless)
    CHARACTER(LEN=80)                                       :: var_name           !! To store variables names for I/O (unitless)

! ==============================================================================================================================

    !! Write restart file for the next simulation from SECHIBA and other modules
    IF (printlev>=3) WRITE (numout,*) 'Write restart file'

    !! 1. Call diffuco_finalize to write restart files
    CALL diffuco_finalize (kjit, kjpindex, rest_id, rstruct )
    
    !! 2. Call energy budget to write restart files
    CALL enerbil_finalize (kjit,   kjpindex,    rest_id,            &
                           evapot, evapot_corr, temp_sol, tsol_rad, &
                           qsurf,  fluxsens,    fluxlat,  vevapp )
    
    !! 3. Call hydrology to write restart files
    !! 3.2 Call water balance from CWRR module (11 soil layers) to write restart file
    CALL hydrol_finalize( kjit,           kjpindex, rest_id,  vegstress,  &
         qsintveg,       humrel,   snow,     snow_age, snow_nobio, &
         snow_nobio_age, snowrho,  snowtemp, snowdz,     &
         snowheat,       snowgrain,    &
         drysoil_frac, evap_bare_lim)
    
    !! 4. Call condveg to write surface variables to restart files
    CALL condveg_finalize (kjit, kjpindex, rest_id, z0m, z0h, roughheight)
    
    !! 5. Call soil thermodynamic to write restart files
    CALL thermosoil_finalize (kjit,    kjpindex, rest_id,   gtemp, &
         soilcap, soilflx, lambda_snow, cgrnd_snow, dgrnd_snow)


    !! 6. Add river routing to restart files  
    IF ( river_routing .AND. nbp_glo .GT. 1) THEN
       !! 6.1 Call river routing to write restart files 
       CALL routing_finalize( kjit, kjpindex, rest_id, flood_frac, flood_res )
    ELSE
       !! 6.2 No routing, set variables to zero
       reinfiltration(:) = zero
       returnflow(:) = zero
       irrigation(:) = zero
       flood_frac(:) = zero
       flood_res(:) = zero
    ENDIF
    
    !#481
    !WRITE(numout,*) 'AHAAA: before slowproc_finalize veget_max', veget_max(:,1)

    !! 7. Call slowproc_main to add 'daily' and annual variables to restart file
    CALL slowproc_finalize (kjit,      kjpindex,    rest_id,     index,      &
                            njsc,      height,      veget,       frac_nobio, &
                            veget_max, reinf_slope, assim_param, frac_age,   &
                            circ_class_biomass, circ_class_n,  &
                            lai_per_level, laieff_fit)
    
    IF (printlev>=3) WRITE (numout,*) 'sechiba_finalize done'
    
  END SUBROUTINE sechiba_finalize


!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_init
!!
!>\BRIEF        Dynamic allocation of the variables, the dimensions of the 
!! variables are determined by user-specified settings. 
!! 
!! DESCRIPTION  : The domain size (:: kjpindex) is used to allocate the correct
!! dimensions to all variables in sechiba. Depending on the variable, its 
!! dimensions are also determined by the number of PFT's (::nvm), number of 
!! soil types (::nstm), number of non-vegetative surface types (::nnobio),
!! number of soil levels (::ngrnd), number of soil layers in the hydrological 
!! model (i.e. cwrr) (::nslm). Values for these variables are set in
!! constantes_soil.f90 and constantes_veg.f90.\n
!!
!! Memory is allocated for all Sechiba variables and new indexing tables
!! are build making use of both (::kjpij) and (::kjpindex). New indexing tables 
!! are needed because a single pixel can contain several PFTs, soil types, etc.
!! The new indexing tables have separate indices for the different
!! PFTs, soil types, etc.\n
!!
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory and builds new indexing 
!! variables for later use.
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================ 

  SUBROUTINE sechiba_init (kjit, kjpij, kjpindex, index, rest_id, lalo)

!! 0.1 Input variables
 
    INTEGER(i_std), INTENT (in)                         :: kjit               !! Time step number (unitless)
    INTEGER(i_std), INTENT (in)                         :: kjpij              !! Total size of the un-compressed grid (unitless)
    INTEGER(i_std), INTENT (in)                         :: kjpindex           !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std), INTENT (in)                         :: rest_id            !! _Restart_ file identifier (unitless)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)    :: index              !! Indeces of the points on the map (unitless)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)     :: lalo               !! Geographical coordinates (latitude,longitude) 
                                                                              !! for pixels (degrees)
!! 0.2 Output variables

!! 0.3 Modified variables

!! 0.4 Local variables

    INTEGER(i_std)                                      :: ier                !! Check errors in memory allocation (unitless)
    INTEGER(i_std)                                      :: ji,jv,ilevel,ipts  !! Indeces (unitless)
!_ ==============================================================================================================================

!! 1. Initialize variables 
  
    ! Debug
    ! It is good to leave this in here.  It is only written out once,
    ! it takes almost no time, and it's necessary for identifying a 
    ! problem pixel.
    DO ipts=1,kjpindex
       WRITE(numout,'(A,I6,10F20.10)') 'pixel number to lat/lon: ',ipts,lalo(ipts,1:2)
    ENDDO
    !-
  
    ! Dynamic allocation with user-specified dimensions on first call
    IF (l_first_sechiba) THEN 
       l_first_sechiba=.FALSE.
    ELSE 
       CALL ipslerr_p(3,'sechiba_init',' l_first_sechiba false . we stop ','','')
    ENDIF

    !! Initialize local printlev
    !printlev_loc=get_printlev('sechiba')
    

    !! 1.1 Initialize 3D vegetation indexation table
    ALLOCATE (indexveg(kjpindex*nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexveg','','')

    ALLOCATE (indexlai(kjpindex*(nlai+1)),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexlai','','')

    ALLOCATE (indexsoil(kjpindex*nstm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexsoil','','')

    ALLOCATE (indexnobio(kjpindex*nnobio),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexnobio','','')

    ALLOCATE (indexgrnd(kjpindex*ngrnd),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexgrnd','','')

    ALLOCATE (indexsnow(kjpindex*nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexsnow','','')

    ALLOCATE (indexlayer(kjpindex*nslm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexlayer','','')

    ALLOCATE (indexnbdl(kjpindex*nbdl),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexnbdl','','')

    ALLOCATE (indexalb(kjpindex*2),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for indexalb','','')

    !! 1.2  Initialize 1D array allocation with restartable value
    ALLOCATE (flood_res(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for flood_res','','')
    flood_res(:) = undef_sechiba

    ALLOCATE (flood_frac(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for kjpindex','','')
    flood_frac(:) = undef_sechiba

    ALLOCATE (snow(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snow','','')
    snow(:) = undef_sechiba

    ALLOCATE (snow_age(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snow_age','','')
    snow_age(:) = undef_sechiba

    ALLOCATE (drysoil_frac(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for drysoil_frac','','')

    ALLOCATE (evap_bare_lim(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for evap_bare_lim','','')

    ALLOCATE (evapot(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for evapot','','')
    evapot(:) = undef_sechiba

    ALLOCATE (evapot_corr(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for evapot_corr','','')

    ALLOCATE (humrel(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for humrel','','')
    humrel(:,:) = undef_sechiba

    ALLOCATE (vegstress(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vegstress','','')
    vegstress(:,:) = undef_sechiba

    ALLOCATE (njsc(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for njsc','','')
    njsc(:)= undef_int

    ALLOCATE (soiltile(kjpindex,nstm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for soiltile','','')

    ALLOCATE (reinf_slope(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for reinf_slope','','')

    ALLOCATE (vbeta1(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta1','','')

    ALLOCATE (vbeta4(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta4','','')

    ALLOCATE (vbeta5(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta5','','')

    ALLOCATE (soilcap(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for soilcap','','')

    ALLOCATE (soilflx(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for soilflx','','')

    ALLOCATE (temp_sol(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for temp_sol','','')
    temp_sol(:) = undef_sechiba

    ALLOCATE (qsurf(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for qsurf','','')
    qsurf(:) = undef_sechiba

    !! 1.3 Initialize 2D array allocation with restartable value
    ALLOCATE (qsintveg(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for qsintveg','','')
    qsintveg(:,:) = undef_sechiba

    ALLOCATE (vbeta2(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta2','','')

    ALLOCATE (vbeta3(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta3','','')

    ALLOCATE (vbeta3pot(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta3pot','','')

    ALLOCATE (gsmean(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for gsmean','','')

    ALLOCATE (cimean(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for cimean','','')

    ALLOCATE (gpp(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for gpp','','')
    gpp(:,:) = undef_sechiba
 
    ALLOCATE (temp_growth(kjpindex),stat=ier) 
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for temp_growth','','')
    temp_growth(:) = undef_sechiba 

    ALLOCATE (veget(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for veget','','')
    veget(:,:)=undef_sechiba

    ALLOCATE (veget_max(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for veget_max','','')

    ALLOCATE (tot_bare_soil(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for tot_bare_soil','','')

    ALLOCATE (lai(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for lai','','')
    lai(:,:)=undef_sechiba

    ALLOCATE (laieff_fit(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for laieff_fit','','')
    CALL laieff_type_init(kjpindex, nlevels_tot, laieff_fit)

    ALLOCATE (frac_age(kjpindex,nvm,nleafages),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for frac_age','','')
    frac_age(:,:,:)=undef_sechiba

    ALLOCATE (height(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for height','','')
    height(:,:)=undef_sechiba

    ALLOCATE (frac_nobio(kjpindex,nnobio),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for frac_nobio','','')
    frac_nobio(:,:) = undef_sechiba

    ALLOCATE (albedo_pft(kjpindex,nvm,2),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for albedo_pft','','')

    ALLOCATE (snow_nobio(kjpindex,nnobio),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snow_nobio','','')
    snow_nobio(:,:) = undef_sechiba

    ALLOCATE (snow_nobio_age(kjpindex,nnobio),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snow_nobio_age','','')
    snow_nobio_age(:,:) = undef_sechiba

    ALLOCATE (assim_param(kjpindex,nvm,npco2),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for assim_param','','')

    !! 1.4 Initialize 1D array allocation 
    ALLOCATE (vevapflo(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vevapflo','','')
    vevapflo(:)=zero

    ALLOCATE (vevapsno(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vevapsno','','')

    ALLOCATE (vevapnu(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vevapnu','','')

    ALLOCATE (totfrac_nobio(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for totfrac_nobio','','')

    ALLOCATE (floodout(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for floodout','','')

    ALLOCATE (runoff(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for runoff','','')

    ALLOCATE (drainage(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for drainage','','')

    ALLOCATE (returnflow(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for returnflow','','')
    returnflow(:) = zero

    ALLOCATE (reinfiltration(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for reinfiltration','','')
    reinfiltration(:) = zero

    ALLOCATE (irrigation(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for irrigation','','')
    irrigation(:) = zero

    ALLOCATE (z0h(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for z0h','','')

    ALLOCATE (z0m(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for z0m','','')

    ALLOCATE (roughheight(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for roughheight','','')

    ALLOCATE (emis(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for emis','','')

    ALLOCATE (tot_melt(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for tot_melt','','')

    ALLOCATE (vbeta(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vbeta','','')

    ALLOCATE (rau(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for rau','','')

    ALLOCATE (deadleaf_cover(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for deadleaf_cover','','')

    ALLOCATE (stempdiag(kjpindex, nbdl),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for stempdiag','','')

    ALLOCATE (co2_flux(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for co2_flux','','')
    co2_flux(:,:)=zero

    ALLOCATE (shumdiag(kjpindex,nbdl),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for shumdiag','','')
    
    ALLOCATE (shumdiag_perma(kjpindex,nbdl),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for shumdiag_perma','','')

    ALLOCATE (litterhumdiag(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for litterhumdiag','','')

    ALLOCATE (ptnlev1(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for ptnlev1','','')

    ALLOCATE (k_litt(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for k_litt','','')

    !! 1.5 Initialize 2D array allocation
    ALLOCATE (vevapwet(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for vevapwet','','')
    vevapwet(:,:)=undef_sechiba

    ALLOCATE (transpir(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for transpir','','')

    ALLOCATE (transpot(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for transpot','','')

    ALLOCATE (transpir_mod(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for transpir_mod','','')
    transpir_mod(:,:) = zero 


    ALLOCATE (stressed(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for stressed','','')
    stressed(:,:) = zero

    ALLOCATE (unstressed(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for unstressed','','')
    unstressed(:,:) = zero

    ALLOCATE (transpir_supply(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for transpir_supply','','')
    transpir_supply(:,:) = zero

    ALLOCATE (vir_transpir_supply(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for vir_transpir_supply','','')
    vir_transpir_supply(:,:) = zero

    ALLOCATE (transpir_supply_column(nlevels_tot,kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for transpir_supply_column','','')
    transpir_supply_column(:,:,:) = zero

    ALLOCATE (e_frac(kjpindex,nvm,nbdl,nstm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for e_frac','','')
    e_frac(:,:,:,:) = zero

    ALLOCATE (qsintmax(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for qsintmax','','')

    ALLOCATE (rveget(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for rveget','','')

    ALLOCATE (rstruct(kjpindex,nvm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for rstruct','','')

    ALLOCATE (pgflux(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for pgflux','','')
    pgflux(:)= 0.0

    ALLOCATE (cgrnd_snow(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for cgrnd_snow','','')
    cgrnd_snow(:,:) = 0

    ALLOCATE (dgrnd_snow(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for dgrnd_snow','','')
    dgrnd_snow(:,:) = 0

    ALLOCATE (lambda_snow(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for lambda_snow','','')
    lambda_snow(:) = 0

    ALLOCATE (temp_sol_add(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for temp_sol_add','','')

    ALLOCATE (gtemp(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for gtemp','','')

    ALLOCATE (frac_snow_veg(kjpindex),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for frac_snow_veg','','')
    
    ALLOCATE (frac_snow_nobio(kjpindex,nnobio),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for frac_snow_nobio','','')

    ALLOCATE (snowrho(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snowrho','','')

    ALLOCATE (snowheat(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snowheat','','')

    ALLOCATE (snowgrain(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snowgrain','','')

    ALLOCATE (snowtemp(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snowtemp','','')

    ALLOCATE (snowdz(kjpindex,nsnow),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for snowdz','','')

    ALLOCATE (mc_layh(kjpindex, nslm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for mc_layh','','')

    ALLOCATE (mcl_layh(kjpindex, nslm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for mcl_layh','','')

    ALLOCATE (tmc_layh(kjpindex, nslm),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for tmc_layh','','')

    ALLOCATE(max_height_store(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p(3,'sechiba_init','Pb in alloc for max_height_store','','')

    ALLOCATE (warnings(kjpindex,nvm,nwarns),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in warnings allocation. We stop. We need kjpindex x nvm x nwarns words = ',&
            & kjpindex,' x ' ,nvm, ' x ',nwarns,' = ',kjpindex*nvm*nwarns
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    ! We aren't restarting the warnings at the moment.
    warnings(:,:,:)=zero

    !! 1.5b Initialize 3D array allocation for albedo
    ALLOCATE (Isotrop_Abs_Tot_p(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init', &
         'Pb in alloc for Isotrop_Abs_Tot_p','','')
    Isotrop_Abs_Tot_p(:,:,:)= zero

    ALLOCATE (Isotrop_Tran_Tot_p(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init', &
         'Pb in alloc for Isotrop_Tran_Tot_p','','')
    Isotrop_Tran_Tot_p(:,:,:)= zero

    ALLOCATE (laieff_isotrop(nlevels_tot,kjpindex,nvm),stat=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init', &
         'Pb in alloc for laieff_isotrop','','')
    laieff_isotrop(:,:,:)= zero


    !! 1.6 Initialize indexing table for the vegetation fields. 
    ! In SECHIBA we work on reduced grids but to store in the full 3D filed vegetation variable 
    ! we need another index table : indexveg, indexsoil, indexnobio and indexgrnd
    DO ji = 1, kjpindex
       !
       DO jv = 1, nlai+1
          indexlai((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, nvm
          indexveg((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !      
       DO jv = 1, nstm
          indexsoil((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !      
       DO jv = 1, nnobio
          indexnobio((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, ngrnd
          indexgrnd((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, nsnow
          indexsnow((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij
       ENDDO

       DO jv = 1, nbdl
          indexnbdl((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij
       ENDDO

       DO jv = 1, nslm
          indexlayer((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, 2
          indexalb((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
    ENDDO

    ! 1.7 now we create and initialize some plant variables that need to be passed
    ! around sechiba and stomate (from the merge, there might be more to
    ! put here)


    ALLOCATE(nlevels_loc(kjpindex),stat=ier)
    IF (ier .NE. 0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for nlevels_loc','','')
    nlevels_loc(:) = val_exp

    ALLOCATE (h_array_out(kjpindex,nvm,ncirc,nlevels_tot),STAT=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for h_arrat_out','','')

    ALLOCATE (z_array_out(kjpindex,nvm,ncirc,nlevels_tot),STAT=ier)
    IF (ier.NE.0) CALL ipslerr_p (3,'sechiba_init','Pb in alloc for z_array_out','','')

    ALLOCATE (profile_vbeta3(kjpindex,nvm,nlevels_tot),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in profile_vbeta3 allocation.'
       WRITE (numout,*) 'We stop. We need kjpindex*nvm words =',kjpindex*nvm*nlevels_tot
       STOP
    END IF

    ALLOCATE (profile_rveget(kjpindex,nvm,nlevels_tot),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in profile_rveget allocation.'
       WRITE (numout,*) 'We stop. We need kjpindex*nvm words =',kjpindex*nvm*nlevels_tot
       STOP
    END IF

        ALLOCATE (delta_c13_assim(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in delta_c13_assim allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
       STOP 'sechiba_init'
    END IF
    delta_c13_assim(:,:)=undef_sechiba

    ALLOCATE (leaf_ci_out(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in leaf_ci_out allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
       STOP 'sechiba_init'
    END IF
    leaf_ci_out(:,:)=undef_sechiba

    ALLOCATE (gpp_day(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in gpp_day allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
       STOP 'sechiba_init'
    END IF
    gpp_day(:,:)=undef_sechiba

    ALLOCATE(circ_class_n(kjpindex,nvm,ncirc),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for circ_class_n. We stop. We need kjpindex*nvm*ncirc words', &
       &      kjpindex,nvm,ncirc
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    circ_class_n(:,:,:) = val_exp

    ALLOCATE(circ_class_biomass(kjpindex,nvm,ncirc,nparts,nelements),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for circ_class_biomass.'
       WRITE(numout,*) 'We stop. We need kjpindex*nvm*nparts*ncirc*nelmements words', &
       &      kjpindex,nvm,ncirc,nparts,nelements
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    circ_class_biomass(:,:,:,:,:) = val_exp


    !ALLOCATE(biomass(kjpindex,nvm,nparts,nelements),stat=ier)
    !IF (ier .NE. 0) THEN
    !   WRITE(numout,*) 'Memory allocation error for biomass.'
    !   WRITE(numout,*) 'We stop. We need kjpindex*nvm*nparts*nelmements words',&
    !   &      kjpindex,nvm,nparts,nelements
    !  CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    !ENDIF
    !biomass(:,:,:,:) = val_exp

    !ALLOCATE(ind(kjpindex,nvm),stat=ier)
    !IF (ier .NE. 0) THEN
    !   WRITE(numout,*) 'Memory allocation error for biomass.'
    !   WRITE(numout,*) 'We stop. We need kjpindex*nvm words', &
    !   &      kjpindex,nvm
    !  CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    !ENDIF
    !ind(:,:) = val_exp

    ALLOCATE(lai_per_level(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for lai_per_level. We stop. '//&
            'We need kjpindex*nvm*nlevels_tot words', &
       &      kjpindex,nvm,nlevels_tot
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    lai_per_level(:,:,:)=val_exp

   !multilayer allocations

    ALLOCATE (u_speed(jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in u_speed allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (flux_rn_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_rn_grid allocation. We stop. We need kjpindex words = ',jnlvls
        STOP 'sechiba_init'
    END IF

    ALLOCATE (flux_h_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_h_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (flux_le_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_le_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (u_speed_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in u_speed_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (temp_atmos_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in temp_atmos_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (q_atmos_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in q_atmos_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (temp_leaf_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in temp_leaf_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (t_a_next_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in t_a_next_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (q_a_next_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in q_a_next_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (swc(kjpindex,nslm,nstm))
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in swc allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (ksave(kjpindex,nslm,nstm))
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in ksave allocation. We stop. We need kjpindexwords = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (t2m_month_out(kjpindex),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in t2m_month_out allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF

    ALLOCATE (psold(kjpindex),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in psold allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF

    ALLOCATE (qsol_sat(kjpindex),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qsol_sat allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF

    ALLOCATE (pdqsold(kjpindex),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in pdqsold allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF

    ALLOCATE (netrad(kjpindex),STAT=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in netrad allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF




!! 2. Read the default value that will be put into variable which are not in the restart file
    CALL ioget_expval(val_exp)
    
    IF (printlev>=3) WRITE (numout,*) ' sechiba_init done '

  END SUBROUTINE sechiba_init
  

!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_clear
!!
!>\BRIEF        Deallocate memory of sechiba's variables
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================ 

  SUBROUTINE sechiba_clear()

!! 1. Initialize first run

    l_first_sechiba=.TRUE.

!! 2. Deallocate dynamic variables of sechiba

    IF ( ALLOCATED (indexveg)) DEALLOCATE (indexveg)
    IF ( ALLOCATED (indexlai)) DEALLOCATE (indexlai)
    IF ( ALLOCATED (indexsoil)) DEALLOCATE (indexsoil)
    IF ( ALLOCATED (indexnobio)) DEALLOCATE (indexnobio)
    IF ( ALLOCATED (indexsnow)) DEALLOCATE (indexsnow)
    IF ( ALLOCATED (indexgrnd)) DEALLOCATE (indexgrnd)
    IF ( ALLOCATED (indexlayer)) DEALLOCATE (indexlayer)
    IF ( ALLOCATED (indexnbdl)) DEALLOCATE (indexnbdl)
    IF ( ALLOCATED (indexalb)) DEALLOCATE (indexalb)
    IF ( ALLOCATED (flood_res)) DEALLOCATE (flood_res)
    IF ( ALLOCATED (flood_frac)) DEALLOCATE (flood_frac)
    IF ( ALLOCATED (snow)) DEALLOCATE (snow)
    IF ( ALLOCATED (snow_age)) DEALLOCATE (snow_age)
    IF ( ALLOCATED (drysoil_frac)) DEALLOCATE (drysoil_frac)
    IF ( ALLOCATED (evap_bare_lim)) DEALLOCATE (evap_bare_lim)
    IF ( ALLOCATED (evapot)) DEALLOCATE (evapot)
    IF ( ALLOCATED (evapot_corr)) DEALLOCATE (evapot_corr)
    IF ( ALLOCATED (humrel)) DEALLOCATE (humrel)
    IF ( ALLOCATED (vegstress)) DEALLOCATE (vegstress)
    IF ( ALLOCATED (soiltile)) DEALLOCATE (soiltile)
    IF ( ALLOCATED (njsc)) DEALLOCATE (njsc)
    IF ( ALLOCATED (reinf_slope)) DEALLOCATE (reinf_slope)
    IF ( ALLOCATED (vbeta1)) DEALLOCATE (vbeta1)
    IF ( ALLOCATED (vbeta4)) DEALLOCATE (vbeta4)
    IF ( ALLOCATED (vbeta5)) DEALLOCATE (vbeta5)
    IF ( ALLOCATED (soilcap)) DEALLOCATE (soilcap)
    IF ( ALLOCATED (soilflx)) DEALLOCATE (soilflx)
    IF ( ALLOCATED (temp_sol)) DEALLOCATE (temp_sol)
    IF ( ALLOCATED (qsurf)) DEALLOCATE (qsurf)
    IF ( ALLOCATED (qsintveg)) DEALLOCATE (qsintveg)
    IF ( ALLOCATED (vbeta2))  DEALLOCATE (vbeta2)
    IF ( ALLOCATED (vbeta3)) DEALLOCATE (vbeta3)
    IF ( ALLOCATED (vbeta3pot)) DEALLOCATE (vbeta3pot)
    IF ( ALLOCATED (gsmean)) DEALLOCATE (gsmean)
    IF ( ALLOCATED (cimean)) DEALLOCATE (cimean)
    IF ( ALLOCATED (gpp)) DEALLOCATE (gpp)
    IF ( ALLOCATED (temp_growth)) DEALLOCATE (temp_growth) 
    IF ( ALLOCATED (veget)) DEALLOCATE (veget)
    IF ( ALLOCATED (veget_max)) DEALLOCATE (veget_max)
    IF ( ALLOCATED (tot_bare_soil)) DEALLOCATE (tot_bare_soil)
    IF ( ALLOCATED (lai)) DEALLOCATE (lai)
    IF ( ALLOCATED (frac_age)) DEALLOCATE (frac_age)
    IF ( ALLOCATED (height)) DEALLOCATE (height)
    IF ( ALLOCATED (roughheight)) DEALLOCATE (roughheight)
    IF ( ALLOCATED (frac_nobio)) DEALLOCATE (frac_nobio)
    IF ( ALLOCATED (snow_nobio)) DEALLOCATE (snow_nobio)
    IF ( ALLOCATED (snow_nobio_age)) DEALLOCATE (snow_nobio_age)
    IF ( ALLOCATED (assim_param)) DEALLOCATE (assim_param)
    IF ( ALLOCATED (vevapflo)) DEALLOCATE (vevapflo)
    IF ( ALLOCATED (vevapsno)) DEALLOCATE (vevapsno)
    IF ( ALLOCATED (vevapnu)) DEALLOCATE (vevapnu)
    IF ( ALLOCATED (totfrac_nobio)) DEALLOCATE (totfrac_nobio)
    IF ( ALLOCATED (floodout)) DEALLOCATE (floodout)
    IF ( ALLOCATED (runoff)) DEALLOCATE (runoff)
    IF ( ALLOCATED (drainage)) DEALLOCATE (drainage)
    IF ( ALLOCATED (reinfiltration)) DEALLOCATE (reinfiltration)
    IF ( ALLOCATED (irrigation)) DEALLOCATE (irrigation)
    IF ( ALLOCATED (tot_melt)) DEALLOCATE (tot_melt)
    IF ( ALLOCATED (vbeta)) DEALLOCATE (vbeta)
    IF ( ALLOCATED (rau)) DEALLOCATE (rau)
    IF ( ALLOCATED (deadleaf_cover)) DEALLOCATE (deadleaf_cover)
    IF ( ALLOCATED (stempdiag)) DEALLOCATE (stempdiag)
    IF ( ALLOCATED (co2_flux)) DEALLOCATE (co2_flux)
    IF ( ALLOCATED (shumdiag)) DEALLOCATE (shumdiag)
    IF ( ALLOCATED (shumdiag_perma)) DEALLOCATE (shumdiag_perma)
    IF ( ALLOCATED (litterhumdiag)) DEALLOCATE (litterhumdiag)
    IF ( ALLOCATED (ptnlev1)) DEALLOCATE (ptnlev1)
    IF ( ALLOCATED (k_litt)) DEALLOCATE (k_litt)
    IF ( ALLOCATED (vevapwet)) DEALLOCATE (vevapwet)
    IF ( ALLOCATED (transpir)) DEALLOCATE (transpir)
    IF ( ALLOCATED (stressed)) DEALLOCATE (stressed)
    IF ( ALLOCATED (unstressed)) DEALLOCATE (unstressed)
    IF ( ALLOCATED (transpir_mod)) DEALLOCATE (transpir_mod)
    IF ( ALLOCATED (transpir_supply)) DEALLOCATE (transpir_supply)
    IF ( ALLOCATED (vir_transpir_supply)) DEALLOCATE (vir_transpir_supply)
    IF ( ALLOCATED (e_frac)) DEALLOCATE (e_frac)
    IF ( ALLOCATED (transpot)) DEALLOCATE (transpot)
    IF ( ALLOCATED (qsintmax)) DEALLOCATE (qsintmax)
    IF ( ALLOCATED (rveget)) DEALLOCATE (rveget)
    IF ( ALLOCATED (rstruct)) DEALLOCATE (rstruct)
    IF ( ALLOCATED (frac_snow_veg)) DEALLOCATE (frac_snow_veg)
    IF ( ALLOCATED (frac_snow_nobio)) DEALLOCATE (frac_snow_nobio)
    IF ( ALLOCATED (snowrho)) DEALLOCATE (snowrho)
    IF ( ALLOCATED (snowgrain)) DEALLOCATE (snowgrain)
    IF ( ALLOCATED (snowtemp)) DEALLOCATE (snowtemp)
    IF ( ALLOCATED (snowdz)) DEALLOCATE (snowdz)
    IF ( ALLOCATED (snowheat)) DEALLOCATE (snowheat)
    IF ( ALLOCATED (cgrnd_snow)) DEALLOCATE (cgrnd_snow)
    IF ( ALLOCATED (dgrnd_snow)) DEALLOCATE (dgrnd_snow)
    IF ( ALLOCATED (lambda_snow)) DEALLOCATE(lambda_snow) 
    IF ( ALLOCATED (gtemp)) DEALLOCATE (gtemp)
    IF ( ALLOCATED (pgflux)) DEALLOCATE (pgflux)
    IF ( ALLOCATED (mc_layh)) DEALLOCATE (mc_layh)
    IF ( ALLOCATED (mcl_layh)) DEALLOCATE (mcl_layh)
    IF ( ALLOCATED (tmc_layh)) DEALLOCATE (tmc_layh)
   
    ! (added for the MERGE)
    IF ( ALLOCATED (albedo_pft)) DEALLOCATE(albedo_pft)
    IF ( ALLOCATED (u_speed)) DEALLOCATE(u_speed)
    IF ( ALLOCATED (warnings)) DEALLOCATE (warnings)
    IF ( ALLOCATED (lai_per_level)) DEALLOCATE(lai_per_level)
    IF ( ALLOCATED (laieff_fit)) DEALLOCATE(laieff_fit)
    IF ( ALLOCATED (max_height_store)) DEALLOCATE(max_height_store)
    IF ( ALLOCATED (h_array_out)) DEALLOCATE(h_array_out)
    IF ( ALLOCATED (z_array_out)) DEALLOCATE(z_array_out)
    IF ( ALLOCATED (Isotrop_Abs_Tot_p)) DEALLOCATE(Isotrop_Abs_Tot_p)
    IF ( ALLOCATED (Isotrop_Tran_Tot_p)) DEALLOCATE(Isotrop_Tran_Tot_p)
    IF ( ALLOCATED (laieff_isotrop)) DEALLOCATE(laieff_isotrop)
    IF ( ALLOCATED (circ_class_biomass)) DEALLOCATE(circ_class_biomass)
    IF ( ALLOCATED (circ_class_n)) DEALLOCATE(circ_class_n)

    IF ( ALLOCATED (profile_vbeta3)) DEALLOCATE(profile_vbeta3)
    IF ( ALLOCATED (profile_rveget)) DEALLOCATE(profile_rveget)
    IF ( ALLOCATED (delta_c13_assim)) DEALLOCATE (delta_c13_assim)
    IF ( ALLOCATED (leaf_ci_out)) DEALLOCATE (leaf_ci_out)

    IF (ALLOCATED(swc)) DEALLOCATE(swc)
    IF (ALLOCATED(ksave)) DEALLOCATE(ksave)

!! 3. Clear all allocated memory

    CALL pft_parameters_clear
    CALL slowproc_clear 
    CALL diffuco_clear 
    CALL enerbil_clear  
    CALL hydrol_clear 
    CALL thermosoil_clear
    CALL condveg_clear 
    CALL routing_clear

  END SUBROUTINE sechiba_clear


!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_var_init
!!
!>\BRIEF        Calculate air density as a function of air temperature and 
!! pressure for each terrestrial pixel.
!! 
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): air density (::rau, kg m^{-3}).
!! 
!! REFERENCE(S)	: None
!! 
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE sechiba_var_init (kjpindex, rau, pb, temp_air) 

!! 0.1 Input variables

    INTEGER(i_std), INTENT (in)                    :: kjpindex        !! Domain size - terrestrial pixels only (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)  :: pb              !! Surface pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)  :: temp_air        !! Air temperature (K)
    
!! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out) :: rau             !! Air density @tex $(kg m^{-3})$ @endtex

!! 0.3 Modified variables

!! 0.4 Local variables

    INTEGER(i_std)                                 :: ji              !! Indices (unitless)
!_ ================================================================================================================================
    
!! 1. Calculate intial air density (::rau)
   
    DO ji = 1,kjpindex
       rau(ji) = pa_par_hpa * pb(ji) / (cte_molr*temp_air(ji))
    END DO

    IF (printlev>=3) WRITE (numout,*) ' sechiba_var_init done '

  END SUBROUTINE sechiba_var_init


!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_end
!!
!>\BRIEF        Swap old for newly calculated soil temperature.
!! 
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): soil temperature (::temp_sol; K)
!! 
!! REFERENCE(S)	: None
!! 
!! FLOWCHART    : None
!! \n
!! ================================================================================================================================ 

  SUBROUTINE sechiba_end (kjpindex, temp_sol_new, temp_sol)
                         

!! 0.1 Input variables

    INTEGER(i_std), INTENT (in)                       :: kjpindex           !! Domain size - terrestrial pixels only (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)     :: temp_sol_new       !! New soil temperature (K)
    
    !! 0.2 Output variables
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)    :: temp_sol           !! Soil temperature (K)

!_ ================================================================================================================================
    
!! 1. Swap temperature

    temp_sol(:) = temp_sol_new(:)
    
    IF (printlev>=3) WRITE (numout,*) ' sechiba_end done '

  END SUBROUTINE sechiba_end

!! ===============================================================================================================================\n
!! SUBROUTINE   : sechiba_interface_orchidee_inca
!!
!>\BRIEF        make the interface between surface and atmospheric chemistry
!! 
!! DESCRIPTION  : This subroutine is called from INCA, the atmospheric chemistry model. It is used to transfer variables from ORCHIDEE to INCA. 
!!
!! RECENT CHANGE(S): move from chemistry module to be more generic (feb - 2017)
!! 
!! MAIN OUTPUT VARIABLE(S): emission COV to be transport by orchidee to inca in fields_out array 
!! 
!! REFERENCE(S) : None
!! 
!! FLOWCHART    : None
!! \n
!!
!! ================================================================================================================================ 
  SUBROUTINE sechiba_interface_orchidee_inca( &
       nvm_out, veget_max_out, veget_frac_out, lai_out, snow_out, &
       field_out_names, fields_out, field_in_names, fields_in)


    INTEGER, INTENT(out)                      :: nvm_out            !! Number of vegetation types
    REAL(r_std), DIMENSION (:,:), INTENT(out) :: veget_max_out      !! Max. fraction of vegetation type (LAI -> infty)
    REAL(r_std), DIMENSION (:,:), INTENT(out) :: veget_frac_out     !! Fraction of vegetation type (unitless, 0-1)  
    REAL(r_std), DIMENSION (:,:), INTENT(out) :: lai_out            !! Surface foliere
    REAL(r_std), DIMENSION (:)  , INTENT(out) :: snow_out           !! Snow mass [Kg/m^2]

    !
    ! Optional arguments
    !
    ! Names and fields for emission variables : to be transport by Orchidee to Inca
    CHARACTER(LEN=*),DIMENSION(:), OPTIONAL, INTENT(IN) :: field_out_names
    REAL(r_std),DIMENSION(:,:,:), OPTIONAL, INTENT(OUT) :: fields_out
    !
    ! Names and fields for deposit variables : to be transport from chemistry model by INCA to ORCHIDEE.
    CHARACTER(LEN=*),DIMENSION(:), OPTIONAL, INTENT(IN) :: field_in_names
    REAL(r_std),DIMENSION(:,:), OPTIONAL, INTENT(IN)    :: fields_in


    ! Variables always transmitted from sechiba to inca
    nvm_out = nvm 
    veget_max_out(:,:)  = veget_max(:,:) 
    veget_frac_out(:,:) = veget(:,:) 
    lai_out(:,:)  = lai(:,:) 
    snow_out(:)  = snow(:) 

    ! Call chemistry_flux_interface if at least one of variables field_out_names or
    ! field_in_names is present in the argument list of sechiba_interface_orchidee_inca when called from inca.
    IF (PRESENT(field_out_names) .AND. .NOT. PRESENT(field_in_names)) THEN 
       CALL chemistry_flux_interface(field_out_names=field_out_names, fields_out=fields_out)
    ELSE IF (.NOT. PRESENT(field_out_names) .AND. PRESENT(field_in_names)) THEN 
       CALL chemistry_flux_interface(field_in_names=field_in_names, fields_in=fields_in)
    ELSE IF (PRESENT(field_out_names) .AND. PRESENT(field_in_names)) THEN 
       CALL chemistry_flux_interface(field_out_names=field_out_names, fields_out=fields_out, &
            field_in_names=field_in_names, fields_in=fields_in)
    ENDIF

  END SUBROUTINE sechiba_interface_orchidee_inca

END MODULE sechiba
